!classDefinition: #CantSuspend category: #'CodigoRepetido-Ejercicio'!
Error subclass: #CantSuspend
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CodigoRepetido-Ejercicio'!


!classDefinition: #NotFound category: #'CodigoRepetido-Ejercicio'!
Error subclass: #NotFound
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CodigoRepetido-Ejercicio'!


!classDefinition: #CustomerBookTest category: #'CodigoRepetido-Ejercicio'!
TestCase subclass: #CustomerBookTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CodigoRepetido-Ejercicio'!

!CustomerBookTest methodsFor: 'testing' stamp: 'MEV 4/4/2019 19:37:00'!
test01AddingCustomerShouldNotTakeMoreThan50Milliseconds

	| customerBook |
	
	customerBook := CustomerBook new.
	
	self should: [customerBook addCustomerNamed: 'John Lennon'] notTakeMoreThan: 50*millisecond .
	
! !

!CustomerBookTest methodsFor: 'testing' stamp: 'MEV 4/6/2019 20:43:11'!
test02RemovingCustomerShouldNotTakeMoreThan100Milliseconds

	| customerBook paulMcCartney |
	
	customerBook := CustomerBook new.
	paulMcCartney := 'Paul McCartney'.
	
	customerBook addCustomerNamed: paulMcCartney.
	 
	self should: [customerBook removeCustomerNamed: paulMcCartney] notTakeMoreThan: 100*millisecond .
	
	

! !

!CustomerBookTest methodsFor: 'testing' stamp: 'MEV 4/4/2019 20:27:09'!
test03CanNotAddACustomerWithEmptyName 

	| customerBook |
			
	customerBook := CustomerBook new.
	
	self canNot: [customerBook addCustomerNamed: ''] 
		onException: Error 
		checkPostcondition: [ :anError | 
			self assert: anError messageText = CustomerBook customerCanNotBeEmptyErrorMessage.
			self assert: customerBook isEmpty ].! !

!CustomerBookTest methodsFor: 'testing' stamp: 'MEV 4/6/2019 21:03:37'!
test04CanNotRemoveAnInvalidCustomer
	
	| customerBook johnLennon |
			
	customerBook := CustomerBook new.
	johnLennon := 'John Lennon'.
	customerBook addCustomerNamed: johnLennon.
	
	self canNot: [customerBook removeCustomerNamed: 'Paul McCartney'] 
		onException: NotFound 
		checkPostcondition: [ :anError | 
			self assert: customerBook numberOfCustomers = 1.
			self assert: (customerBook includesCustomerNamed: johnLennon)].! !

!CustomerBookTest methodsFor: 'testing' stamp: 'MEV 4/6/2019 21:51:32'!
test05SuspendingACustomerShouldNotRemoveItFromCustomerBook

	| customerBook paulMcCartney|
	
	customerBook := CustomerBook new.
	paulMcCartney := 'Paul McCartney'.
	
	customerBook addCustomerNamed: paulMcCartney.
	customerBook suspendCustomerNamed: paulMcCartney.
	
	self numbersOfCustomersIn: customerBook areActive: 0 suspended: 1 andTotal:1.
	self assert: (customerBook includesCustomerNamed: paulMcCartney).
	"
	self assert: 0 equals: customerBook numberOfActiveCustomers.
	self assert: 1 equals: customerBook numberOfSuspendedCustomers.
	self assert: 1 equals: customerBook numberOfCustomers.
	self assert: (customerBook includesCustomerNamed: paulMcCartney).
	"

	

! !

!CustomerBookTest methodsFor: 'testing' stamp: 'MEV 4/6/2019 20:38:39'!
test06RemovingASuspendedCustomerShouldRemoveItFromCustomerBook

	| customerBook paulMcCartney|
	
	customerBook := CustomerBook new.
	paulMcCartney := 'Paul McCartney'.
	
	customerBook addCustomerNamed: paulMcCartney.
	customerBook suspendCustomerNamed: paulMcCartney.
	customerBook removeCustomerNamed: paulMcCartney.
	
	self numbersOfCustomersIn: customerBook areActive: 0 suspended: 0 andTotal:0.
	self deny: (customerBook includesCustomerNamed: paulMcCartney).
	
	"self assert: 0 equals: customerBook numberOfActiveCustomers.
	self assert: 0 equals: customerBook numberOfSuspendedCustomers.
	self assert: 0 equals: customerBook numberOfCustomers.
	self deny: (customerBook includesCustomerNamed: paulMcCartney).
"

	
! !

!CustomerBookTest methodsFor: 'testing' stamp: 'MEV 4/6/2019 21:51:40'!
test07CanNotSuspendAnInvalidCustomer
	
	| customerBook johnLennon |
			
	customerBook := CustomerBook new.
	johnLennon := 'John Lennon'.
	customerBook addCustomerNamed: johnLennon.
	
	self canNot: [customerBook suspendCustomerNamed: 'Paul McCartney'] onCustomerBook: customerBook andIsStillACustomer:johnLennon.
	"
	[ customerBook suspendCustomerNamed: 'Paul McCartney'.
	self fail ]
		on: CantSuspend 
		do: [ :anError | 
			self assert: customerBook numberOfCustomers = 1.
			self assert: (customerBook includesCustomerNamed: johnLennon) ]
"! !

!CustomerBookTest methodsFor: 'testing' stamp: 'MEV 4/6/2019 21:51:46'!
test08CanNotSuspendAnAlreadySuspendedCustomer
	
	| customerBook johnLennon |
			
	customerBook := CustomerBook new.
	johnLennon := 'John Lennon'.
	customerBook addCustomerNamed: johnLennon.
	customerBook suspendCustomerNamed: johnLennon.
	
	self canNot: [customerBook suspendCustomerNamed: johnLennon] onCustomerBook: customerBook andIsStillACustomer:johnLennon.
	
	"
	[ customerBook suspendCustomerNamed: johnLennon.
	self fail ]
		on: CantSuspend 
		do: [ :anError | 
			self assert: customerBook numberOfCustomers = 1.
			self assert: (customerBook includesCustomerNamed: johnLennon) ]
"! !


!CustomerBookTest methodsFor: 'auxiliar' stamp: 'MEV 4/6/2019 20:26:56'!
canNot: aBlock onCustomerBook: customerBook andIsStillACustomer:aCustomer
	
	[ aBlock value.
	self fail ]
		on: CantSuspend 
		do: [ :anError | 
			self assert: customerBook numberOfCustomers = 1.
			self assert: (customerBook includesCustomerNamed: aCustomer) ]
! !

!CustomerBookTest methodsFor: 'auxiliar' stamp: 'MEV 4/6/2019 20:27:05'!
canNot: aBlock onException: type checkPostcondition:aClosure
	
	[ aBlock value.
	self fail ]
		on: type 
		do: aClosure.! !

!CustomerBookTest methodsFor: 'auxiliar' stamp: 'MEV 4/6/2019 20:27:22'!
numbersOfCustomersIn: customerBook areActive: activeNumber suspended: suspendedNumber andTotal:customerNumber
	
	self assert: activeNumber equals: customerBook numberOfActiveCustomers.
	self assert: suspendedNumber equals: customerBook numberOfSuspendedCustomers.
	self assert: customerNumber equals: customerBook numberOfCustomers.
! !

!CustomerBookTest methodsFor: 'auxiliar' stamp: 'MEV 4/6/2019 20:27:47'!
should: aBlock notTakeMoreThan: millisecondsAmount

	|millisecondsBeforeRunning millisecondsAfterRunning|
	
	millisecondsBeforeRunning := Time millisecondClockValue * millisecond.
	aBlock value.
	millisecondsAfterRunning := Time millisecondClockValue * millisecond.
	
	self assert: (millisecondsAfterRunning-millisecondsBeforeRunning) < millisecondsAmount 
	
! !


!classDefinition: #CustomerBook category: #'CodigoRepetido-Ejercicio'!
Object subclass: #CustomerBook
	instanceVariableNames: 'suspended active'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CodigoRepetido-Ejercicio'!

!CustomerBook methodsFor: 'testing' stamp: 'MEV 4/6/2019 20:28:20'!
includesCustomerNamed: aName

	^(active includes: aName) or: [ suspended includes: aName ]! !

!CustomerBook methodsFor: 'testing' stamp: 'MEV 4/6/2019 20:28:26'!
isEmpty
	
	^active isEmpty and: [ suspended isEmpty ]! !


!CustomerBook methodsFor: 'initialization' stamp: 'MEV 4/6/2019 20:28:12'!
initialize

	super initialize.
	active := OrderedCollection new.
	suspended:= OrderedCollection new.! !


!CustomerBook methodsFor: 'auxiliares' stamp: 'MEV 4/7/2019 15:37:50'!
numberOfCustomersFrom: aList

	^aList size.! !

!CustomerBook methodsFor: 'auxiliares' stamp: 'MEV 4/7/2019 15:28:32'!
removeCustomerNamed: aName from: anOrderedCollection
	
	1 to: anOrderedCollection size do: 
	[ :index |
		aName = (anOrderedCollection at: index)
			ifTrue: [
				anOrderedCollection removeAt: index.
				^ aName 
			] 
	].! !

!CustomerBook methodsFor: 'auxiliares' stamp: 'MEV 4/7/2019 15:36:51'!
signalCustomer: aBlock

	^self error: (aBlock value)! !


!CustomerBook methodsFor: 'customer management' stamp: 'MEV 4/6/2019 20:31:25'!
addCustomerNamed: aName

	aName isEmpty ifTrue: [ self signalCustomerNameCannotBeEmpty ].
	(self includesCustomerNamed: aName) ifTrue: [ self signalCustomerAlreadyExists ].
	
	active add: aName ! !

!CustomerBook methodsFor: 'customer management' stamp: 'MEV 4/7/2019 15:38:06'!
numberOfActiveCustomers
	
	^self numberOfCustomersFrom: active.! !

!CustomerBook methodsFor: 'customer management' stamp: 'MEV 4/7/2019 15:39:03'!
numberOfCustomers
	
	^(self numberOfCustomersFrom: active) + (self numberOfCustomersFrom: suspended).! !

!CustomerBook methodsFor: 'customer management' stamp: 'MEV 4/7/2019 15:38:21'!
numberOfSuspendedCustomers
	
	^self numberOfCustomersFrom: suspended.! !

!CustomerBook methodsFor: 'customer management' stamp: 'MEV 4/7/2019 15:30:16'!
removeCustomerNamed: aName 
	
	|activeCustomer suspendedCustomer|
	
	activeCustomer := self removeCustomerNamed: aName from: active.
	suspendedCustomer := self removeCustomerNamed: aName from: suspended. 
	((activeCustomer = aName) | (suspendedCustomer  = aName)) ifTrue:[^aName].
	
	^NotFound signal.! !

!CustomerBook methodsFor: 'customer management' stamp: 'MEV 4/7/2019 15:34:57'!
signalCustomerAlreadyExists 

	self signalCustomer: [self class customerAlreadyExistsErrorMessage].! !

!CustomerBook methodsFor: 'customer management' stamp: 'MEV 4/7/2019 15:35:56'!
signalCustomerNameCannotBeEmpty 

	self signalCustomer: [self class customerCanNotBeEmptyErrorMessage ].! !

!CustomerBook methodsFor: 'customer management' stamp: 'MEV 4/6/2019 20:29:22'!
suspendCustomerNamed: aName 
	
	(active includes: aName) ifFalse: [^CantSuspend signal].
	
	active remove: aName.
	
	suspended add: aName

! !

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

!classDefinition: 'CustomerBook class' category: #'CodigoRepetido-Ejercicio'!
CustomerBook class
	instanceVariableNames: ''!

!CustomerBook class methodsFor: 'error messages' stamp: 'NR 4/3/2019 10:16:22'!
customerAlreadyExistsErrorMessage

	^'Customer Already Exists'! !

!CustomerBook class methodsFor: 'error messages' stamp: 'NR 4/3/2019 10:16:31'!
customerCanNotBeEmptyErrorMessage

	^'Customer Name Cannot Be Empty'! !
