!classDefinition: #OOStackTest category: #'Stack-Exercise'!
TestCase subclass: #OOStackTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Stack-Exercise'!

!OOStackTest methodsFor: 'test' stamp: 'MEV 4/16/2019 17:35:16'!
test01StackShouldBeEmptyWhenCreated

	| stack |
	
	stack := OOStack new.
	
	self assert: stack isEmpty! !

!OOStackTest methodsFor: 'test' stamp: 'MEV 4/16/2019 18:36:43'!
test02PushAddElementsToTheStack

	| stack |
	
	stack := OOStack new.
	stack push: 'Something'.
	
	self deny: stack isEmpty! !

!OOStackTest methodsFor: 'test' stamp: 'MEV 4/16/2019 17:35:54'!
test03PopRemovesElementsFromTheStack

	| stack |
	
	stack := OOStack new.
	stack push: 'Something'.
	stack pop.
	
	self assert: stack isEmpty! !

!OOStackTest methodsFor: 'test' stamp: 'MEV 4/17/2019 15:27:08'!
test04PopReturnsLastPushedObject

	| stack pushedObject |
	
	stack := OOStack new.
	pushedObject := 'Something'.
	stack push: pushedObject.
	
	self assert: stack pop = pushedObject! !

!OOStackTest methodsFor: 'test' stamp: 'MEV 4/17/2019 20:28:43'!
test05StackBehavesLIFO

	| stack firstPushedObject secondPushedObject |
	
	stack := OOStack new.
	firstPushedObject := 'First'.
	secondPushedObject := 'Second'.
	
	stack push: firstPushedObject.
	stack push: secondPushedObject.
	
	self assert: stack pop = secondPushedObject.
	self assert: stack pop = firstPushedObject.
	self assert: stack isEmpty 

	! !

!OOStackTest methodsFor: 'test' stamp: 'MEV 4/17/2019 17:54:46'!
test06TopReturnsLastPushedObject

	| stack pushedObject |
	
	stack := OOStack new.
	pushedObject := 'Something'.
	
	stack push: pushedObject.
	
	self assert: stack top = pushedObject.
	! !

!OOStackTest methodsFor: 'test' stamp: 'MEV 4/17/2019 17:59:43'!
test07TopDoesNotRemoveObjectFromStack

	| stack pushedObject |
	
	stack := OOStack new.
	pushedObject := 'Something'.
	
	stack push: pushedObject.
	
	self assert: stack size = 1.
	stack top.
	self assert: stack size = 1.

	! !

!OOStackTest methodsFor: 'test' stamp: 'MEV 4/16/2019 19:02:09'!
test08CanNotPopWhenThereAreNoObjectsInTheStack

	| stack  |
	
	stack := OOStack new.
	self
		should: [ stack pop ]
		raise: Error - MessageNotUnderstood 
		withExceptionDo: [ :anError |
			self assert: anError messageText = OOStack stackEmptyErrorDescription ]
		
! !

!OOStackTest methodsFor: 'test' stamp: 'HAW 4/14/2017 22:48:36'!
test09CanNotPopWhenThereAreNoObjectsInTheStackAndTheStackHadObjects

	| stack  |
	
	stack := OOStack new.
	stack push: 'Something'.
	stack pop.
	
	self
		should: [ stack pop ]
		raise: Error - MessageNotUnderstood 
		withExceptionDo: [ :anError |
			self assert: anError messageText = OOStack stackEmptyErrorDescription ]
		
! !

!OOStackTest methodsFor: 'test' stamp: 'HAW 4/14/2017 22:48:44'!
test10CanNotTopWhenThereAreNoObjectsInTheStack

	| stack  |
	
	stack := OOStack new.
	self
		should: [ stack top ]
		raise: Error - MessageNotUnderstood 
		withExceptionDo: [ :anError |
			self assert: anError messageText = OOStack stackEmptyErrorDescription ]
		
! !


!classDefinition: #SentenceFinderByPrefixTest category: #'Stack-Exercise'!
TestCase subclass: #SentenceFinderByPrefixTest
	instanceVariableNames: 'emptyPrefix prefixWithSpaces wintPrefix prefixWint winter emptyStack winterIsComing isNotWinter winterWithUpperCaseW stack'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Stack-Exercise'!

!SentenceFinderByPrefixTest methodsFor: 'tests' stamp: 'MEV 4/18/2019 19:13:03'!
test01PrefixCantBeEmpty

	|sentenceFinder|
	
	sentenceFinder  := SentenceFinderByPrefix with:stack .
		
	self  evaluate:  [ sentenceFinder find: emptyPrefix.] AndAssertErrorDescription: 
	SentenceFinderByPrefix emptyPrefixErrorDescription .	 	
 ! !

!SentenceFinderByPrefixTest methodsFor: 'tests' stamp: 'MEV 4/18/2019 19:13:08'!
test02PrefixCantContainSpaces

	|sentenceFinder|
	
	sentenceFinder  := SentenceFinderByPrefix with:stack .
	
	self evaluate:  [ sentenceFinder find: prefixWithSpaces.] AndAssertErrorDescription:
	 SentenceFinderByPrefix prefixWithSpacesErrorDescription . 
 ! !

!SentenceFinderByPrefixTest methodsFor: 'tests' stamp: 'MEV 4/18/2019 19:44:25'!
test03NoSentencesAreFoundInEmptyStack


	|sentenceFinder sentencesWithPrefix sentencesFound|
	
	sentenceFinder  := SentenceFinderByPrefix with:stack.
	
	sentencesWithPrefix  := (Bag new).
	sentencesFound  := sentenceFinder find: wintPrefix .
	
	self assert: (sentencesWithPrefix = sentencesFound).! !

!SentenceFinderByPrefixTest methodsFor: 'tests' stamp: 'MEV 4/19/2019 21:36:43'!
test04wintIsPrefixOfwinterAndwinterIsComingAndNotOfIsNotWinter

	|sentenceFinder sentencesWithPrefix sentencesFound|
	
	stack push: winter.
	stack push: isNotWinter.
	stack push: winterIsComing .
	
	sentenceFinder  := SentenceFinderByPrefix with:stack .
	sentencesWithPrefix  := Bag with:winter with:winterIsComing .
	sentencesFound  := sentenceFinder find: wintPrefix .
	
	self assert: (sentencesWithPrefix = sentencesFound).! !

!SentenceFinderByPrefixTest methodsFor: 'tests' stamp: 'MEV 4/19/2019 21:37:07'!
test05SentenceFinderByPrefixIsCaseSensitive

	|sentenceFinder sentencesWithPrefix sentencesFound|
	
	stack push: winterWithUpperCaseW.
	stack push: winter .
	
	sentenceFinder  := SentenceFinderByPrefix with:stack .
	sentencesWithPrefix  := Bag with:winter .
	sentencesFound  := sentenceFinder find: wintPrefix .
	
	self assert: (sentencesWithPrefix = sentencesFound).! !

!SentenceFinderByPrefixTest methodsFor: 'tests' stamp: 'MEV 4/19/2019 21:32:30'!
test06StackIsNotModified

	|sentenceFinder stackCopy|
	
	
	stack push:winter.
	stack push: winterWithUpperCaseW.
	stack push: isNotWinter.
	stack push: winterIsComing.
	
	stackCopy := stack veryDeepCopy.
	
	sentenceFinder  := SentenceFinderByPrefix with:stack .
	sentenceFinder find: wintPrefix .
	
	self assert: (self stack:(sentenceFinder stack) isEqualTo: stackCopy).! !


!SentenceFinderByPrefixTest methodsFor: 'auxiliares' stamp: 'MEV 4/18/2019 17:12:58'!
evaluate: aBlock AndAssertErrorDescription: anErrorDescription

	self should: aBlock
		raise: Error - MessageNotUnderstood 
		withExceptionDo: [ :anError |
			self assert: anError messageText = anErrorDescription . ].! !

!SentenceFinderByPrefixTest methodsFor: 'auxiliares' stamp: 'MEV 4/19/2019 21:23:14'!
stack:aStack isEqualTo: anotherStack

	
	|stackCopy|
	
	stackCopy := aStack veryDeepCopy.
	
	(aStack size = anotherStack size)ifFalse:[^false].
	
	[stackCopy isEmpty]whileFalse:[(stackCopy pop = anotherStack pop)ifFalse:[^false].].
	^true.
	! !


!SentenceFinderByPrefixTest methodsFor: 'setup' stamp: 'MEV 4/19/2019 21:39:15'!
setUp

	winter := 'winter'.
	winterIsComing := 'winter is coming'.
	winterWithUpperCaseW := 'Winter'.
	isNotWinter := 'is not winter'.
	
	stack := OOStack new.
	
	
	emptyPrefix := ''.
	prefixWithSpaces := 'prefix with spaces'.
	wintPrefix := 'wint'.
	! !


!classDefinition: #OOStack category: #'Stack-Exercise'!
Object subclass: #OOStack
	instanceVariableNames: 'stack'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Stack-Exercise'!

!OOStack methodsFor: 'accessing' stamp: 'MEV 4/17/2019 18:27:48'!
pop
	
	^(stack last) removeLastFrom:self.

! !

!OOStack methodsFor: 'accessing' stamp: 'MEV 4/17/2019 18:13:58'!
push: aString 
	
	stack addLast: (OOStackNodeWithValue with: aString).! !

!OOStack methodsFor: 'accessing' stamp: 'MEV 4/17/2019 18:14:01'!
size

	^stack size - 1.! !

!OOStack methodsFor: 'accessing' stamp: 'MEV 4/17/2019 22:24:14'!
top
	
	
	^(stack last) top.
	! !


!OOStack methodsFor: 'auxiliares' stamp: 'MEV 4/17/2019 18:27:02'!
removeLast

	^stack removeLast value.! !


!OOStack methodsFor: 'testing' stamp: 'MEV 4/17/2019 18:14:09'!
isEmpty

	^stack last isEmpty .! !


!OOStack methodsFor: 'initialize' stamp: 'MEV 4/20/2019 17:22:37'!
initialize

	stack := LinkedList new.
	stack addLast: (OOStackBaseNode basicNew).! !

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

!classDefinition: 'OOStack class' category: #'Stack-Exercise'!
OOStack class
	instanceVariableNames: ''!

!OOStack class methodsFor: 'error descriptions' stamp: 'HernanWilkinson 5/7/2012 11:51'!
stackEmptyErrorDescription
	
	^ 'Stack is empty'! !


!classDefinition: #OOStackBaseNode category: #'Stack-Exercise'!
OOStack subclass: #OOStackBaseNode
	instanceVariableNames: 'nextLink'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Stack-Exercise'!

!OOStackBaseNode methodsFor: 'accessing' stamp: 'MEV 4/17/2019 17:41:37'!
nextLink

	^nextLink.! !

!OOStackBaseNode methodsFor: 'accessing' stamp: 'MEV 4/17/2019 17:41:48'!
nextLink: aLink 

	^nextLink _ aLink! !

!OOStackBaseNode methodsFor: 'accessing' stamp: 'MEV 4/17/2019 18:25:03'!
remove: anOOStack

	self error: OOStack stackEmptyErrorDescription.! !

!OOStackBaseNode methodsFor: 'accessing' stamp: 'MEV 4/17/2019 18:26:37'!
removeLastFrom: anOOStack

	self error: OOStack stackEmptyErrorDescription.! !

!OOStackBaseNode methodsFor: 'accessing' stamp: 'MEV 4/17/2019 17:51:01'!
top

	self error: OOStack stackEmptyErrorDescription.! !


!OOStackBaseNode methodsFor: 'testing' stamp: 'MEV 4/17/2019 17:46:49'!
isEmpty

	^true.! !


!classDefinition: #OOStackNodeWithValue category: #'Stack-Exercise'!
OOStack subclass: #OOStackNodeWithValue
	instanceVariableNames: 'nextLink value'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Stack-Exercise'!

!OOStackNodeWithValue methodsFor: 'initialize' stamp: 'MEV 4/17/2019 15:22:19'!
initializeWith: anObject

	value:=anObject .! !


!OOStackNodeWithValue methodsFor: 'accessing' stamp: 'MEV 4/17/2019 15:19:31'!
nextLink

	^nextLink.! !

!OOStackNodeWithValue methodsFor: 'accessing' stamp: 'MEV 4/17/2019 15:19:45'!
nextLink: aLink 

	^nextLink _ aLink! !

!OOStackNodeWithValue methodsFor: 'accessing' stamp: 'MEV 4/17/2019 18:28:11'!
removeLastFrom: anOOStack

	^anOOStack removeLast.! !

!OOStackNodeWithValue methodsFor: 'accessing' stamp: 'MEV 4/17/2019 17:58:17'!
top

	^self value.! !

!OOStackNodeWithValue methodsFor: 'accessing' stamp: 'MEV 4/17/2019 15:19:56'!
value

	^value.! !

!OOStackNodeWithValue methodsFor: 'accessing' stamp: 'MEV 4/17/2019 15:20:33'!
value: anObject

	value := anObject.! !


!OOStackNodeWithValue methodsFor: 'testing' stamp: 'MEV 4/17/2019 17:48:37'!
isEmpty

	^false.! !

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

!classDefinition: 'OOStackNodeWithValue class' category: #'Stack-Exercise'!
OOStackNodeWithValue class
	instanceVariableNames: ''!

!OOStackNodeWithValue class methodsFor: 'instance creation' stamp: 'MEV 4/17/2019 15:21:27'!
with: anObject

	^self new initializeWith: anObject.! !


!classDefinition: #SentenceFinderByPrefix category: #'Stack-Exercise'!
Object subclass: #SentenceFinderByPrefix
	instanceVariableNames: 'stack'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Stack-Exercise'!

!SentenceFinderByPrefix methodsFor: 'initialize' stamp: 'MEV 4/17/2019 20:34:06'!
initializeWith: anOOStack 
	
		stack := anOOStack.! !


!SentenceFinderByPrefix methodsFor: 'accessing' stamp: 'MEV 4/19/2019 21:50:57'!
find: aPrefix 
		
		|stackCopy|

		stackCopy := stack veryDeepCopy .
		
		self checkPrefixForEmptinessOrForSpaces: aPrefix.
		
		^self sentencesFromStack: stackCopy thatStartWithPrefix: aPrefix.
		
		! !

!SentenceFinderByPrefix methodsFor: 'accessing' stamp: 'MEV 4/19/2019 21:42:58'!
sentencesFromStack: stackCopy thatStartWithPrefix: aPrefix

		|sentencesFound sentence|
		
		sentencesFound := Bag new.
		
		[stackCopy isEmpty not] whileTrue: 
			[sentence := stackCopy pop.
			(sentence is:aPrefix substringAt: 1)ifTrue:[sentencesFound add:sentence.].
			].
		^sentencesFound
		
		! !

!SentenceFinderByPrefix methodsFor: 'accessing' stamp: 'MEV 4/18/2019 18:48:45'!
stack

	^stack.! !


!SentenceFinderByPrefix methodsFor: 'testing' stamp: 'MEV 4/19/2019 21:51:18'!
checkPrefixForEmptinessOrForSpaces: aPrefix.
		
	(aPrefix isEmpty)ifTrue:[self error: SentenceFinderByPrefix emptyPrefixErrorDescription .].
	(aPrefix includesSubString: ' ')ifTrue:[self error: SentenceFinderByPrefix prefixWithSpacesErrorDescription .].

		
		! !

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

!classDefinition: 'SentenceFinderByPrefix class' category: #'Stack-Exercise'!
SentenceFinderByPrefix class
	instanceVariableNames: ''!

!SentenceFinderByPrefix class methodsFor: 'error descriptions' stamp: 'MEV 4/17/2019 20:31:48'!
emptyPrefixErrorDescription

	^'The prefix can not be empty'! !

!SentenceFinderByPrefix class methodsFor: 'error descriptions' stamp: 'MEV 4/17/2019 20:42:57'!
prefixWithSpacesErrorDescription

	^'Prefix can not contain spaces'.! !

!SentenceFinderByPrefix class methodsFor: 'error descriptions' stamp: 'MEV 4/17/2019 21:44:41'!
stackIsEmptyErrorDescription

	^'Stack can not be empty'.! !


!SentenceFinderByPrefix class methodsFor: 'instance creation' stamp: 'MEV 4/17/2019 20:33:13'!
with: anOOStack 
	
		^self new initializeWith: anOOStack.! !
