!classDefinition: #AccountTransferTest category: #PortfolioTreePrinter!
TestCase subclass: #AccountTransferTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PortfolioTreePrinter'!

!AccountTransferTest methodsFor: 'tests' stamp: 'MEV 6/2/2019 18:52:30'!
test01TransferBetweenTwoAccountsModifiesTheirBalanceCorrectly

	|anAccount anotherAccount|
	
	anAccount := ReceptiveAccount new.
	Deposit register: 100 on: anAccount.
	
	anotherAccount := ReceptiveAccount new.
	Deposit register: 200 on: anotherAccount.
	
	AccountTransfer for:50 from:anAccount to:anotherAccount.
	
	self assert:(anAccount balance) equals:50.
	self assert:(anotherAccount balance) equals:250.! !

!AccountTransferTest methodsFor: 'tests' stamp: 'MEV 6/2/2019 18:53:01'!
test02AWithdrawLegFromATransferTransactionKnowsItsDepositLegAndViceversa

	|anAccount anotherAccount aTransfer aWithdrawLeg aDepositLeg|
	
	anAccount := ReceptiveAccount new.
	Deposit register: 100 on: anAccount.
	
	anotherAccount := ReceptiveAccount new.
	Deposit register: 200 on: anotherAccount.
	
	aTransfer :=AccountTransfer for:50 from:anAccount to:anotherAccount.
	aDepositLeg := aTransfer depositLeg .
	aWithdrawLeg := aTransfer withdrawLeg .
	
	self assert:(aDepositLeg withdrawLeg) equals:aWithdrawLeg .
	self assert:(aWithdrawLeg depositLeg ) equals:aDepositLeg .! !

!AccountTransferTest methodsFor: 'tests' stamp: 'MEV 6/2/2019 18:57:27'!
test03CanNotMakeATransferTransactionBetweenAnAccountAndItself

	|anAccount|
	
	anAccount := ReceptiveAccount new.
	Deposit register: 100 on: anAccount.
	
	self should: [ AccountTransfer for:50 from:anAccount to:anAccount.]
		raise: Error
		withExceptionDo: [ :anError |
			self assert: AccountTransfer canNotMakeATransferFromOneAccountToItselfErrorSignal equals: anError messageText.
			self assert: (anAccount balance) equals:100.
			 ]! !


!classDefinition: #PortfolioTest category: #PortfolioTreePrinter!
TestCase subclass: #PortfolioTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PortfolioTreePrinter'!

!PortfolioTest methodsFor: 'tests' stamp: 'HAW 5/25/2019 11:16:26'!
test01BalanceOfPortfolioWithoutAccountsIsZero

	self assert: 0 equals: Portfolio new balance! !

!PortfolioTest methodsFor: 'tests' stamp: 'HAW 5/25/2019 11:17:51'!
test02BalanceOfPortfolioWithOneAccountIsAccountBalance

	| account portfolio |
	
	account := ReceptiveAccount new.
	Deposit register: 100 on: account.
	portfolio := Portfolio with: account.
	
	self assert: account balance equals: portfolio balance! !

!PortfolioTest methodsFor: 'tests' stamp: 'HAW 5/25/2019 11:23:25'!
test03BalanceOfPortfolioIsCalculatedRecursivelyOnPortfolios

	| simplePortfolioAccount simplePortfolio composedPortfolioAccount composedPortofolio |
	
	simplePortfolioAccount := ReceptiveAccount new.
	Deposit register: 100 on: simplePortfolioAccount.
	simplePortfolio := Portfolio with: simplePortfolioAccount.
	
	composedPortfolioAccount := ReceptiveAccount new.
	Withdraw register: 50 on: composedPortfolioAccount.
	composedPortofolio := Portfolio with: simplePortfolio with: composedPortfolioAccount.
	
	self assert: (composedPortfolioAccount balance + simplePortfolio balance) equals: composedPortofolio balance! !

!PortfolioTest methodsFor: 'tests' stamp: 'HAW 5/25/2019 11:43:15'!
test04PortfolioWithoutAccountsHasNoRegisteredTransaction

	self deny: (Portfolio new hasRegistered: (Deposit for: 100))! !

!PortfolioTest methodsFor: 'tests' stamp: 'HAW 5/25/2019 11:43:11'!
test05PortfolioHasRegisteredItsAccountsTransactions

	| account portfolio deposit |
	
	account := ReceptiveAccount new.
	deposit := Deposit register: 100 on: account.
	portfolio := Portfolio with: account.
	
	self assert: (portfolio hasRegistered: deposit)! !

!PortfolioTest methodsFor: 'tests' stamp: 'HAW 5/25/2019 11:58:06'!
test06PortfolioLooksForRegisteredTransactionsRecursively

	| simplePortfolioAccount simplePortfolio composedPortfolioAccount composedPortfolio composedPortfolioAccountWithdraw simplePortfolioAccountDeposit |
	
	simplePortfolioAccount := ReceptiveAccount new.
	simplePortfolioAccountDeposit := Deposit register: 100 on: simplePortfolioAccount.
	simplePortfolio := Portfolio with: simplePortfolioAccount.
	
	composedPortfolioAccount := ReceptiveAccount new.
	composedPortfolioAccountWithdraw := Withdraw register: 50 on: composedPortfolioAccount.
	composedPortfolio := Portfolio with: simplePortfolio with: composedPortfolioAccount.
	
	self assert: (composedPortfolio hasRegistered: simplePortfolioAccountDeposit).
	self assert: (composedPortfolio hasRegistered: composedPortfolioAccountWithdraw)! !

!PortfolioTest methodsFor: 'tests' stamp: 'HAW 5/25/2019 11:58:10'!
test07PortfolioHasNoTransactionWhenHasNoAccounts

	self assert: Portfolio new transactions isEmpty! !

!PortfolioTest methodsFor: 'tests' stamp: 'HAW 5/25/2019 11:58:15'!
test08PortfolioTransasctionsIncludesAllItsAccountsTransactions

	| account portfolio accountDeposit anotherAccount portfolioTransactions anotherAccountWithdraw |
	
	account := ReceptiveAccount new.
	accountDeposit := Deposit register: 100 on: account.
	anotherAccount := ReceptiveAccount new.
	anotherAccountWithdraw := Withdraw register: 100 on: account.
	portfolio := Portfolio with: account.
	
	portfolioTransactions := portfolio transactions.
	
	self assert: 2 equals: portfolioTransactions size.
	self assert: (portfolioTransactions includes: accountDeposit).
	self assert: (portfolioTransactions includes: anotherAccountWithdraw)! !

!PortfolioTest methodsFor: 'tests' stamp: 'HAW 5/25/2019 11:58:20'!
test09PortfolioTransactionsAreCalculatedRecursively

	| simplePortfolioAccount simplePortfolio composedPortfolioAccount composedPortfolio composedPortfolioAccountWithdraw simplePortfolioAccountDeposit composedPortfolioTransactions |
	
	simplePortfolioAccount := ReceptiveAccount new.
	simplePortfolioAccountDeposit := Deposit register: 100 on: simplePortfolioAccount.
	simplePortfolio := Portfolio with: simplePortfolioAccount.
	
	composedPortfolioAccount := ReceptiveAccount new.
	composedPortfolioAccountWithdraw := Withdraw register: 50 on: composedPortfolioAccount.
	composedPortfolio := Portfolio with: simplePortfolio with: composedPortfolioAccount.
	
	composedPortfolioTransactions := composedPortfolio transactions.
	self assert: 2 equals: composedPortfolioTransactions size.
	self assert: (composedPortfolioTransactions includes: simplePortfolioAccountDeposit).
	self assert: (composedPortfolioTransactions includes: composedPortfolioAccountWithdraw)! !

!PortfolioTest methodsFor: 'tests' stamp: 'HAW 5/25/2019 11:58:24'!
test10PortfolioCanNotIncludeTheSameAccountMoreThanOnce

	| account portfolio |
	
	account := ReceptiveAccount new.
	portfolio := Portfolio with: account.
	
	self 
		should: [ portfolio add: account ]
		raise: Error - MessageNotUnderstood 
		withExceptionDo: [ :anError |
			self assert: Portfolio canNotAddAccountErrorMessage equals: anError messageText.
			self assert: 1 equals: portfolio accountsSize.
			self assert: (portfolio accountsIncludes: account) ]! !

!PortfolioTest methodsFor: 'tests' stamp: 'HAW 5/25/2019 11:58:28'!
test11PortfolioCanNotIncludeAccountOfItsPortfolios

	| account simplePortfolio composedPortfolio |
	
	account := ReceptiveAccount new.
	simplePortfolio := Portfolio with: account.
	composedPortfolio := Portfolio with: simplePortfolio.
	
	self 
		should: [ composedPortfolio add: account ]
		raise: Error - MessageNotUnderstood 
		withExceptionDo: [ :anError |
			self assert: Portfolio canNotAddAccountErrorMessage equals: anError messageText.
			self assert: 1 equals: composedPortfolio accountsSize.
			self assert: (composedPortfolio accountsIncludes: simplePortfolio) ]! !

!PortfolioTest methodsFor: 'tests' stamp: 'HAW 5/25/2019 11:58:32'!
test12PortfolioCanNotIncludeItself

	| account simplePortfolio |
	
	account := ReceptiveAccount new.
	simplePortfolio := Portfolio with: account.
	
	self 
		should: [ simplePortfolio add: simplePortfolio ]
		raise: Error - MessageNotUnderstood 
		withExceptionDo: [ :anError |
			self assert: Portfolio canNotAddAccountErrorMessage equals: anError messageText.
			self assert: 1 equals: simplePortfolio accountsSize.
			self assert: (simplePortfolio accountsIncludes: account) ]! !

!PortfolioTest methodsFor: 'tests' stamp: 'HAW 5/25/2019 12:01:51'!
test13ComposedPortfolioCanNotHaveParentPortfolioAccount

	| account simplePortfolio composedPortfolio |
	
	account := ReceptiveAccount new.
	simplePortfolio := Portfolio new.
	composedPortfolio := Portfolio with: simplePortfolio.
	composedPortfolio add: account.
	
	self 
		should: [ simplePortfolio add: account ]
		raise: Error - MessageNotUnderstood 
		withExceptionDo: [ :anError |
			self assert: Portfolio canNotAddAccountErrorMessage equals: anError messageText.
			self assert: simplePortfolio accountsIsEmpty ]! !

!PortfolioTest methodsFor: 'tests' stamp: 'HAW 5/25/2019 12:12:16'!
test14ComposedPortfolioCanNotHaveAccountOfAnyRootParentRecursively

	| account leftParentPortfolio leftRootParentPortfolio portfolio rightParentPortfolio rightRootParentPortfolio |
	
	account := ReceptiveAccount new.
	portfolio := Portfolio new.
	leftParentPortfolio := Portfolio with: portfolio .
	leftRootParentPortfolio := Portfolio with: leftParentPortfolio.
	leftRootParentPortfolio add: account.
	
	rightParentPortfolio := Portfolio with: portfolio .
	rightRootParentPortfolio := Portfolio with: rightParentPortfolio.
	rightRootParentPortfolio add: account.

	self 
		should: [ portfolio add: account ]
		raise: Error - MessageNotUnderstood 
		withExceptionDo: [ :anError |
			self assert: Portfolio canNotAddAccountErrorMessage equals: anError messageText.
			self assert: portfolio accountsIsEmpty ]! !

!PortfolioTest methodsFor: 'tests' stamp: 'HAW 5/29/2019 16:31:18'!
test15PortfolioCanNotIncludeAnyOfTheComposedAccountOfPortfolioToAdd

	| portfolioToAdd portfolioToModify rootPortfolio sharedAccount |
	
	sharedAccount := ReceptiveAccount new.
	portfolioToModify := Portfolio new.
	rootPortfolio := Portfolio with: sharedAccount with: portfolioToModify.
	portfolioToAdd := Portfolio with: sharedAccount.
	
	self 
		should: [ portfolioToModify add: portfolioToAdd ]
		raise: Error - MessageNotUnderstood 
		withExceptionDo: [ :anError | 
			self assert: Portfolio canNotAddAccountErrorMessage equals: anError messageText.
			self assert: portfolioToModify accountsIsEmpty ]! !


!classDefinition: #ReceptiveAccountTest category: #PortfolioTreePrinter!
TestCase subclass: #ReceptiveAccountTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PortfolioTreePrinter'!

!ReceptiveAccountTest methodsFor: 'tests' stamp: 'HAW 5/23/2019 15:19:48'!
test01ReceptiveAccountHaveZeroAsBalanceWhenCreated 

	| account |
	
	account := ReceptiveAccount new.

	self assert: 0 equals: account balance .
! !

!ReceptiveAccountTest methodsFor: 'tests' stamp: 'HAW 5/23/2019 15:19:54'!
test02DepositIncreasesBalanceOnTransactionValue 

	| account |
	
	account := ReceptiveAccount  new.
	Deposit register: 100 on: account.
		
	self assert: 100 equals: account balance .
! !

!ReceptiveAccountTest methodsFor: 'tests' stamp: 'HAW 5/23/2019 15:20:02'!
test03WithdrawDecreasesBalanceOnTransactionValue 

	| account |
	
	account := ReceptiveAccount new.
	Deposit register: 100 on: account.
	Withdraw register: 50 on: account.
		
	self assert: 50 equals: account balance .
! !

!ReceptiveAccountTest methodsFor: 'tests' stamp: 'HAW 5/23/2019 15:20:32'!
test04WithdrawValueMustBePositive 

	| account withdrawValue |
	
	account := ReceptiveAccount new.
	withdrawValue := 50.
	
	self assert: withdrawValue equals: (Withdraw register: withdrawValue on: account) value
! !

!ReceptiveAccountTest methodsFor: 'tests' stamp: 'HAW 5/23/2019 15:20:46'!
test05ReceptiveAccountKnowsRegisteredTransactions 

	| account deposit withdraw |
	
	account := ReceptiveAccount new.
	deposit := Deposit register: 100 on: account.
	withdraw := Withdraw register: 50 on: account.
		
	self assert: (account hasRegistered: deposit).
	self assert: (account hasRegistered: withdraw).
! !

!ReceptiveAccountTest methodsFor: 'tests' stamp: 'HAW 5/23/2019 15:20:54'!
test06ReceptiveAccountDoNotKnowNotRegisteredTransactions

	| account deposit withdraw |
	
	account := ReceptiveAccount new.
	deposit :=  Deposit for: 100.
	withdraw := Withdraw for: 50.
		
	self deny: (account hasRegistered: deposit).
	self deny: (account hasRegistered:withdraw).
! !

!ReceptiveAccountTest methodsFor: 'tests' stamp: 'HAW 5/23/2019 15:21:24'!
test07AccountKnowsItsTransactions 

	| account1 deposit1 |
	
	account1 := ReceptiveAccount new.
	
	deposit1 := Deposit register: 100 on: account1.
		
	self assert: 1 equals: account1 transactions size .
	self assert: (account1 transactions includes: deposit1).
! !


!classDefinition: #ReportTest category: #PortfolioTreePrinter!
TestCase subclass: #ReportTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PortfolioTreePrinter'!

!ReportTest methodsFor: 'tests' stamp: 'MEV 6/2/2019 18:44:54'!
test01ReportOfTwoAccountsWithATransferTransactionBetweenThem

	|anAccount anotherAccount aReport anotherReport|
	
	anAccount := ReceptiveAccount new.
	Deposit register: 100 on: anAccount.
	
	anotherAccount := ReceptiveAccount new.
	Deposit register: 200 on: anotherAccount.
	
	AccountTransfer for:50 from:anAccount to:anotherAccount.
	
	aReport:=Report on: anAccount.
	anotherReport := Report on: anotherAccount .
	
	self assert:(aReport depositAmount) equals:100.
	self assert:(aReport withdrawAmount) equals:0.
	self assert:(aReport depositByTransferAmount) equals:0.
	self assert:(aReport withdrawByTransferAmount) equals:50.
	self assert:(aReport balance) equals:50.
	
	self assert:(anotherReport depositAmount) equals:200.
	self assert:(anotherReport withdrawAmount) equals:0.
	self assert:(anotherReport depositByTransferAmount) equals:50.
	self assert:(anotherReport withdrawByTransferAmount) equals:0.
	self assert:(anotherReport balance) equals:250.! !

!ReportTest methodsFor: 'tests' stamp: 'MEV 6/2/2019 18:45:16'!
test02ReportOfAPortfolioWithAnAccount

	|simplePortfolioAccount simplePortfolio aReport|
	simplePortfolioAccount := ReceptiveAccount new.
	Deposit register: 100 on: simplePortfolioAccount.
	Withdraw register: 50 on: simplePortfolioAccount.
	simplePortfolio := Portfolio with: simplePortfolioAccount.
	
	aReport:=Report on: simplePortfolio.
	
	self assert:(aReport depositAmount) equals:100.
	self assert:(aReport withdrawAmount) equals:50.
	self assert:(aReport depositByTransferAmount) equals:0.
	self assert:(aReport withdrawByTransferAmount) equals:0.
	self assert:(aReport balance) equals:50.
	! !

!ReportTest methodsFor: 'tests' stamp: 'MEV 6/2/2019 18:45:42'!
test03ReportOfAPortfolioComposedOfAnAccountAndAPortfolio

	|simplePortfolioAccount simplePortfolio composedPortfolioAccount composedPortofolio aReport|
	simplePortfolioAccount := ReceptiveAccount new.
	Deposit register: 100 on: simplePortfolioAccount.
	Withdraw register: 50 on: simplePortfolioAccount.
	simplePortfolio := Portfolio with: simplePortfolioAccount.
	
	composedPortfolioAccount := ReceptiveAccount new.
	Withdraw register: 50 on: composedPortfolioAccount.
	composedPortofolio := Portfolio with: simplePortfolio with: composedPortfolioAccount.
	
	aReport:=Report on: composedPortofolio .
	
	self assert:(aReport depositAmount) equals:100.
	self assert:(aReport withdrawAmount) equals:100.
	self assert:(aReport depositByTransferAmount) equals:0.
	self assert:(aReport withdrawByTransferAmount) equals:0.
	self assert:(aReport balance) equals:0.
	! !

!ReportTest methodsFor: 'tests' stamp: 'MEV 6/2/2019 18:46:24'!
test04ReportOfAComposedPortfolioWithATransferTransactionBetweenAccountsOfThatPortfolio

	|simplePortfolioAccount simplePortfolio composedPortfolioAccount composedPortofolio aReport|
	simplePortfolioAccount := ReceptiveAccount new.
	Deposit register: 100 on: simplePortfolioAccount.
	Withdraw register: 50 on: simplePortfolioAccount.
	simplePortfolio := Portfolio with: simplePortfolioAccount.
	
	composedPortfolioAccount := ReceptiveAccount new.
	Withdraw register: 50 on: composedPortfolioAccount.
	composedPortofolio := Portfolio with: simplePortfolio with: composedPortfolioAccount.
	
	AccountTransfer for:50 from:simplePortfolioAccount to:composedPortfolioAccount.
	
	aReport:=Report on: composedPortofolio .
	
	self assert:(aReport depositAmount) equals:100.
	self assert:(aReport withdrawAmount) equals:100.
	self assert:(aReport depositByTransferAmount) equals:50.
	self assert:(aReport withdrawByTransferAmount) equals:50.
	self assert:(aReport balance) equals:0.
	! !

!ReportTest methodsFor: 'tests' stamp: 'MEV 6/2/2019 23:17:46'!
test05ReportShowsTheCorrectInformationOfAPortfolio

	|simplePortfolioAccount simplePortfolio composedPortfolioAccount composedPortofolio aReport aStream aDepositLine aWithdrawLine 
	aDepositByTransferLine aWithdrawByTransferLine aBalanceLine|
	
	simplePortfolioAccount := ReceptiveAccount new.
	Deposit register: 100 on: simplePortfolioAccount.
	Withdraw register: 50 on: simplePortfolioAccount.
	simplePortfolio := Portfolio with: simplePortfolioAccount.
	
	composedPortfolioAccount := ReceptiveAccount new.
	Withdraw register: 50 on: composedPortfolioAccount.
	composedPortofolio := Portfolio with: simplePortfolio with: composedPortfolioAccount.
	
	aReport:=Report on: composedPortofolio .
	
	aStream := ReadStream on:(aReport accountSummary).
	aDepositLine := 'Depsito por 100.'.
	aWithdrawLine := 'Extraccin por 100.'.
	aDepositByTransferLine := 'Entrada por transferencia de 0.'.
	aWithdrawByTransferLine  := 'Salida por transferencia de 0.'.
	aBalanceLine := 'Balance = 0.'.
	
	self assert: aDepositLine equals: (aStream nextLine).
	self assert: aWithdrawLine equals: (aStream nextLine).
	self assert: aWithdrawByTransferLine equals: (aStream nextLine).
	self assert: aDepositByTransferLine equals: (aStream nextLine).
	self assert: aBalanceLine equals: (aStream nextLine).
	
	! !

!ReportTest methodsFor: 'tests' stamp: 'MEV 6/2/2019 23:17:46'!
test06AddingTransactionToPortfolioAfterCreatingAReportModifiesReportWhenShowingIt

	|simplePortfolioAccount simplePortfolio composedPortfolioAccount composedPortofolio aReport aStream aDepositLine aWithdrawLine 
	aDepositByTransferLine aWithdrawByTransferLine aBalanceLine|
	
	simplePortfolioAccount := ReceptiveAccount new.
	Deposit register: 100 on: simplePortfolioAccount.
	Withdraw register: 50 on: simplePortfolioAccount.
	simplePortfolio := Portfolio with: simplePortfolioAccount.
	
	composedPortfolioAccount := ReceptiveAccount new.
	Withdraw register: 50 on: composedPortfolioAccount.
	composedPortofolio := Portfolio with: simplePortfolio with: composedPortfolioAccount.
	
	aReport:=Report on: composedPortofolio .
	
	Deposit register: 100 on: simplePortfolioAccount.
	
	aStream := ReadStream on:(aReport accountSummary).
	aDepositLine := 'Depsito por 200.'.
	aWithdrawLine := 'Extraccin por 100.'.
	aDepositByTransferLine := 'Entrada por transferencia de 0.'.
	aWithdrawByTransferLine  := 'Salida por transferencia de 0.'.
	aBalanceLine := 'Balance = 100.'.
	
	self assert: aDepositLine equals: (aStream nextLine).
	self assert: aWithdrawLine equals: (aStream nextLine).
	self assert: aWithdrawByTransferLine equals: (aStream nextLine).
	self assert: aDepositByTransferLine equals: (aStream nextLine).
	self assert: aBalanceLine equals: (aStream nextLine).
	
	! !

!ReportTest methodsFor: 'tests' stamp: 'MEV 6/2/2019 23:26:40'!
test07TheTransferNetOfOneAccountIsTheNegatedValueOfTheTransferNetOfTheOtherAccount

	|anAccount anotherAccount aReport anotherReport|
	
	anAccount := ReceptiveAccount new.
	Deposit register: 100 on: anAccount.
	
	anotherAccount := ReceptiveAccount new.
	Deposit register: 200 on: anotherAccount.
	
	AccountTransfer for:50 from:anAccount to:anotherAccount.
	
	aReport:=Report on: anAccount.
	anotherReport := Report on: anotherAccount .
	
	self assert:(aReport transferNet )equals:(-50).
	self assert:(anotherReport transferNet )equals:50.! !

!ReportTest methodsFor: 'tests' stamp: 'MEV 6/3/2019 10:37:55'!
test08TransferNetOfPortfolioWithTransfersBetweenItsAccountsItsZero

	|simplePortfolioAccount simplePortfolio composedPortfolioAccount composedPortofolio aReport |
	
	simplePortfolioAccount := ReceptiveAccount new.
	Deposit register: 100 on: simplePortfolioAccount.
	Withdraw register: 50 on: simplePortfolioAccount.
	simplePortfolio := Portfolio with: simplePortfolioAccount.
	
	composedPortfolioAccount := ReceptiveAccount new.
	Withdraw register: 50 on: composedPortfolioAccount.
	composedPortofolio := Portfolio with: simplePortfolio with: composedPortfolioAccount.
	
	AccountTransfer for:50 from:simplePortfolioAccount to:composedPortfolioAccount.
	
	aReport:=Report on: composedPortofolio .
	
	self assert:(aReport transferNet )equals:0.
	
	
	
	
	! !

!ReportTest methodsFor: 'tests' stamp: 'MEV 6/3/2019 10:40:02'!
test09TransferingBetweenAccountsAfterReportIsMadeModifiesTheReport

	|anAccount simplePortfolioAccount simplePortfolio composedPortfolioAccount composedPortofolio aReport |
	
	anAccount := ReceptiveAccount new.
	Deposit register: 100 on: anAccount .
	
	simplePortfolioAccount := ReceptiveAccount new.
	Deposit register: 100 on: simplePortfolioAccount.
	Withdraw register: 50 on: simplePortfolioAccount.
	simplePortfolio := Portfolio with: simplePortfolioAccount.
	
	composedPortfolioAccount := ReceptiveAccount new.
	Withdraw register: 50 on: composedPortfolioAccount.
	composedPortofolio := Portfolio with: simplePortfolio with: composedPortfolioAccount.
	
	AccountTransfer for:50 from:simplePortfolioAccount to:composedPortfolioAccount.
	
	aReport:=Report on: composedPortofolio .
	
	self assert:(aReport transferNet )equals:0.
	
	AccountTransfer for:50 from:simplePortfolioAccount to:anAccount .
	
	self assert:(aReport transferNet )equals:(-50).
	
	
	! !


!classDefinition: #Account category: #PortfolioTreePrinter!
Object subclass: #Account
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PortfolioTreePrinter'!

!Account methodsFor: 'testing' stamp: 'HAW 5/25/2019 12:23:47'!
hasRegistered: aTransaction

	self subclassResponsibility ! !

!Account methodsFor: 'testing' stamp: 'HAW 5/25/2019 12:24:25'!
isComposedBy: anAccount

	self subclassResponsibility ! !


!Account methodsFor: 'balance' stamp: 'HAW 5/25/2019 12:23:40'!
balance

	self subclassResponsibility ! !


!Account methodsFor: 'transactions' stamp: 'HAW 5/25/2019 12:23:27'!
addTransactionsTo: aCollectionOfTransactions

	self subclassResponsibility ! !

!Account methodsFor: 'transactions' stamp: 'HAW 5/25/2019 12:23:15'!
transactions

	self subclassResponsibility ! !


!Account methodsFor: 'composition' stamp: 'HAW 5/25/2019 12:24:04'!
addedTo: aPortfolio

	self subclassResponsibility ! !


!classDefinition: #Portfolio category: #PortfolioTreePrinter!
Account subclass: #Portfolio
	instanceVariableNames: 'accounts parents'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PortfolioTreePrinter'!

!Portfolio methodsFor: 'accounts management' stamp: 'HAW 5/25/2019 11:49:20'!
accountsIncludes: anAccount

	^accounts includes: anAccount ! !

!Portfolio methodsFor: 'accounts management' stamp: 'HAW 5/25/2019 12:05:04'!
accountsIsEmpty
	
	^accounts isEmpty ! !

!Portfolio methodsFor: 'accounts management' stamp: 'HAW 5/25/2019 11:49:06'!
accountsSize
	
	^accounts size! !

!Portfolio methodsFor: 'accounts management' stamp: 'HAW 5/25/2019 12:19:20'!
add: accountToAdd

	self assertCanAdd: accountToAdd.
		
	accounts add: accountToAdd.
	accountToAdd addedTo: self 
	! !

!Portfolio methodsFor: 'accounts management' stamp: 'HAW 5/25/2019 12:17:31'!
rootParents
	
	| rootParents |
	
	rootParents := Set new.
	self addRootParentsTo: rootParents.
	
	^ rootParents! !


!Portfolio methodsFor: 'initialization' stamp: 'HAW 5/25/2019 12:03:18'!
initialize

	accounts := OrderedCollection new.
	parents := OrderedCollection new.! !


!Portfolio methodsFor: 'balance' stamp: 'HAW 5/25/2019 11:19:36'!
balance
	
	^accounts sum: [ :anAccount | anAccount balance ] ifEmpty: [ 0 ]! !


!Portfolio methodsFor: 'transactions' stamp: 'HAW 5/25/2019 11:42:55'!
addTransactionsTo: aCollectionOfTransactions

	accounts do: [ :anAccount | anAccount addTransactionsTo: aCollectionOfTransactions ]! !

!Portfolio methodsFor: 'transactions' stamp: 'HAW 5/25/2019 11:38:32'!
transactions
	
	| transactions |
	
	transactions := OrderedCollection new.
	accounts do: [ :anAccount | anAccount addTransactionsTo: transactions ].
	
	^transactions ! !


!Portfolio methodsFor: 'composition' stamp: 'HAW 5/25/2019 12:02:59'!
addedTo: aPortfolio 
	
	parents add: aPortfolio ! !


!Portfolio methodsFor: 'testing' stamp: 'HAW 5/25/2019 12:20:56'!
anyRootParentIsComposedBy: accountToAdd

	^self rootParents anySatisfy: [ :aParent | aParent isComposedBy: accountToAdd]! !

!Portfolio methodsFor: 'testing' stamp: 'HAW 5/25/2019 11:28:29'!
hasRegistered: aTransaction

	^accounts anySatisfy: [ :anAccount | anAccount hasRegistered: aTransaction ]! !

!Portfolio methodsFor: 'testing' stamp: 'HAW 5/29/2019 16:24:54'!
isComposedBy: anAccount

	^ self = anAccount or: [ accounts anySatisfy: [ :composedAccount | (composedAccount isComposedBy: anAccount) or: [ anAccount isComposedBy: composedAccount ]]]! !


!Portfolio methodsFor: 'account management - private' stamp: 'HAW 5/25/2019 12:17:31'!
addRootParentsTo: rootParents

	parents 
		ifEmpty: [ rootParents add: self ] 
		ifNotEmpty: [ parents do: [ :aParent | aParent addRootParentsTo: rootParents ]]! !

!Portfolio methodsFor: 'account management - private' stamp: 'HAW 5/25/2019 12:20:36'!
assertCanAdd: accountToAdd

	(self anyRootParentIsComposedBy: accountToAdd) ifTrue: [ self signalCanNotAddAccount ].
! !

!Portfolio methodsFor: 'account management - private' stamp: 'HAW 5/25/2019 11:48:34'!
signalCanNotAddAccount
	
	self error: self class canNotAddAccountErrorMessage! !

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

!classDefinition: 'Portfolio class' category: #PortfolioTreePrinter!
Portfolio class
	instanceVariableNames: ''!

!Portfolio class methodsFor: 'as yet unclassified' stamp: 'HAW 5/25/2019 11:48:55'!
canNotAddAccountErrorMessage
	
	^'Can not add repeated account to a portfolio'! !

!Portfolio class methodsFor: 'as yet unclassified' stamp: 'HAW 5/25/2019 11:18:21'!
with: anAccount

	^self new 
		add: anAccount;
		yourself! !

!Portfolio class methodsFor: 'as yet unclassified' stamp: 'HAW 5/25/2019 11:23:59'!
with: anAccount with: anotherAccount

	^self new 
		add: anAccount;
		add: anotherAccount;
		yourself! !


!classDefinition: #ReceptiveAccount category: #PortfolioTreePrinter!
Account subclass: #ReceptiveAccount
	instanceVariableNames: 'transactions'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PortfolioTreePrinter'!

!ReceptiveAccount methodsFor: 'initialization' stamp: 'HernanWilkinson 7/13/2011 18:35'!
initialize

	super initialize.
	transactions := OrderedCollection new.! !


!ReceptiveAccount methodsFor: 'transactions' stamp: 'HAW 5/25/2019 11:38:52'!
addTransactionsTo: aCollectionOfTransactions

	aCollectionOfTransactions addAll: transactions ! !

!ReceptiveAccount methodsFor: 'transactions' stamp: 'HernanWilkinson 7/13/2011 18:37'!
register: aTransaction

	transactions add: aTransaction 
! !

!ReceptiveAccount methodsFor: 'transactions' stamp: 'HernanWilkinson 7/13/2011 18:37'!
transactions 

	^ transactions copy! !


!ReceptiveAccount methodsFor: 'balance' stamp: 'HAW 5/25/2019 11:24:46'!
balance

	^transactions 
		inject: 0
		into: [ :currentBalance :transaction | transaction affectBalance: currentBalance ]! !


!ReceptiveAccount methodsFor: 'testing' stamp: 'HAW 10/11/2018 16:53:54'!
hasRegistered: aTtransaction

	^ transactions includes: aTtransaction 
! !

!ReceptiveAccount methodsFor: 'testing' stamp: 'HAW 5/25/2019 11:54:51'!
isComposedBy: anAccount

	^self = anAccount ! !


!ReceptiveAccount methodsFor: 'composition' stamp: 'HAW 5/25/2019 12:03:32'!
addedTo: aPortfolio 
	
	! !


!classDefinition: #AccountTransaction category: #PortfolioTreePrinter!
Object subclass: #AccountTransaction
	instanceVariableNames: 'transfer'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PortfolioTreePrinter'!

!AccountTransaction methodsFor: 'value' stamp: 'HernanWilkinson 9/12/2011 12:25'!
value 

	self subclassResponsibility ! !


!AccountTransaction methodsFor: 'balance' stamp: 'HAW 5/25/2019 11:25:39'!
affectBalance: aBalance

	self subclassResponsibility ! !

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

!classDefinition: 'AccountTransaction class' category: #PortfolioTreePrinter!
AccountTransaction class
	instanceVariableNames: ''!

!AccountTransaction class methodsFor: 'instance creation' stamp: 'HernanWilkinson 7/13/2011 18:38'!
register: aValue on: account

	| withdraw |
	
	withdraw := self for: aValue.
	account register: withdraw.
		
	^ withdraw! !


!classDefinition: #Deposit category: #PortfolioTreePrinter!
AccountTransaction subclass: #Deposit
	instanceVariableNames: 'value'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PortfolioTreePrinter'!

!Deposit methodsFor: 'initialization' stamp: 'HernanWilkinson 7/13/2011 18:45'!
initializeFor: aValue

	value := aValue ! !


!Deposit methodsFor: 'value' stamp: 'HernanWilkinson 7/13/2011 18:38'!
value

	^ value! !


!Deposit methodsFor: 'balance' stamp: 'HAW 5/25/2019 11:25:02'!
affectBalance: aBalance

	^aBalance + value ! !

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

!classDefinition: 'Deposit class' category: #PortfolioTreePrinter!
Deposit class
	instanceVariableNames: ''!

!Deposit class methodsFor: 'instance creation' stamp: 'HernanWilkinson 7/13/2011 18:38'!
for: aValue

	^ self new initializeFor: aValue ! !


!classDefinition: #Withdraw category: #PortfolioTreePrinter!
AccountTransaction subclass: #Withdraw
	instanceVariableNames: 'value'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PortfolioTreePrinter'!

!Withdraw methodsFor: 'initialization' stamp: 'HernanWilkinson 7/13/2011 18:46'!
initializeFor: aValue

	value := aValue ! !


!Withdraw methodsFor: 'value' stamp: 'HernanWilkinson 7/13/2011 18:33'!
value

	^ value! !


!Withdraw methodsFor: 'balance' stamp: 'HAW 5/25/2019 11:25:15'!
affectBalance: aBalance

	^aBalance - value! !

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

!classDefinition: 'Withdraw class' category: #PortfolioTreePrinter!
Withdraw class
	instanceVariableNames: ''!

!Withdraw class methodsFor: 'instance creation' stamp: 'HernanWilkinson 7/13/2011 18:33'!
for: aValue

	^ self new initializeFor: aValue ! !


!classDefinition: #AccountTransfer category: #PortfolioTreePrinter!
Object subclass: #AccountTransfer
	instanceVariableNames: 'value withdrawLeg depositLeg'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PortfolioTreePrinter'!

!AccountTransfer methodsFor: 'legs' stamp: 'MEV 6/2/2019 15:35:10'!
depositLeg

	^depositLeg .! !

!AccountTransfer methodsFor: 'legs' stamp: 'MEV 6/2/2019 15:35:22'!
withdrawLeg

	^withdrawLeg .! !


!AccountTransfer methodsFor: 'value' stamp: 'MEV 6/2/2019 15:41:11'!
value

	^value.! !


!AccountTransfer methodsFor: 'initialize' stamp: 'MEV 6/2/2019 15:48:34'!
initializeFor:anAmount

		value:=anAmount .
		depositLeg := DepositLeg of:self.
		withdrawLeg := WithdrawLeg of:self.! !


!AccountTransfer methodsFor: 'balance' stamp: 'MEV 6/2/2019 15:46:11'!
affectBalance: aBalance

	self subclassResponsibility ! !

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

!classDefinition: 'AccountTransfer class' category: #PortfolioTreePrinter!
AccountTransfer class
	instanceVariableNames: ''!

!AccountTransfer class methodsFor: 'error signals' stamp: 'MEV 6/2/2019 18:57:58'!
canNotMakeATransferFromOneAccountToItselfErrorSignal

	^'Can not transfer money from one account to itself'.! !


!AccountTransfer class methodsFor: 'class initialization' stamp: 'MEV 6/2/2019 15:24:55'!
for: anAmount

	^self new initializeFor:anAmount.! !

!AccountTransfer class methodsFor: 'class initialization' stamp: 'MEV 6/2/2019 19:00:33'!
for:anAmount from:anAccount to:anotherAccount

	|transfer|
	
	(anAccount =anotherAccount )ifTrue: [self error: AccountTransfer canNotMakeATransferFromOneAccountToItselfErrorSignal ].
	transfer:=self for:anAmount .
	anAccount register: transfer withdrawLeg.
	anotherAccount register:transfer depositLeg.
	^transfer.! !


!classDefinition: #DepositLeg category: #PortfolioTreePrinter!
AccountTransfer subclass: #DepositLeg
	instanceVariableNames: 'transfer'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PortfolioTreePrinter'!

!DepositLeg methodsFor: 'initialize' stamp: 'MEV 6/2/2019 15:20:08'!
initializeFor:aTransfer

	transfer:=aTransfer .! !


!DepositLeg methodsFor: 'legs' stamp: 'MEV 6/2/2019 15:49:34'!
depositLeg

	^self.! !

!DepositLeg methodsFor: 'legs' stamp: 'MEV 6/2/2019 15:33:54'!
withdrawLeg

	^transfer withdrawLeg.! !


!DepositLeg methodsFor: 'value' stamp: 'MEV 6/2/2019 15:20:34'!
value

	^transfer value.! !


!DepositLeg methodsFor: 'balance' stamp: 'MEV 6/2/2019 15:23:00'!
affectBalance: aBalance

	^aBalance + (self value). ! !

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

!classDefinition: 'DepositLeg class' category: #PortfolioTreePrinter!
DepositLeg class
	instanceVariableNames: ''!

!DepositLeg class methodsFor: 'initialization' stamp: 'MEV 6/2/2019 15:21:49'!
of: aTransfer

	^self new initializeFor:aTransfer.! !


!classDefinition: #WithdrawLeg category: #PortfolioTreePrinter!
AccountTransfer subclass: #WithdrawLeg
	instanceVariableNames: 'transfer'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PortfolioTreePrinter'!

!WithdrawLeg methodsFor: 'value' stamp: 'MEV 6/2/2019 15:20:48'!
value

	^transfer value.! !


!WithdrawLeg methodsFor: 'legs' stamp: 'MEV 6/2/2019 15:33:32'!
depositLeg

	^transfer depositLeg .! !

!WithdrawLeg methodsFor: 'legs' stamp: 'MEV 6/2/2019 15:49:50'!
withdrawLeg

	^self .! !


!WithdrawLeg methodsFor: 'initialize' stamp: 'MEV 6/2/2019 15:19:22'!
initializeFor:aTransfer

	transfer:=aTransfer .! !


!WithdrawLeg methodsFor: 'balance' stamp: 'MEV 6/2/2019 15:23:16'!
affectBalance: aBalance

	^aBalance - (self value). ! !

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

!classDefinition: 'WithdrawLeg class' category: #PortfolioTreePrinter!
WithdrawLeg class
	instanceVariableNames: ''!

!WithdrawLeg class methodsFor: 'initialization' stamp: 'MEV 6/2/2019 15:21:49'!
of: aTransfer

	^self new initializeFor:aTransfer.! !


!classDefinition: #Report category: #PortfolioTreePrinter!
Object subclass: #Report
	instanceVariableNames: 'report depositAmount withdrawAmount depositByTransferAmount withdrawByTransferAmount account'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PortfolioTreePrinter'!

!Report methodsFor: 'summary' stamp: 'MEV 6/2/2019 23:21:15'!
accountSummary

	self calculateTransactionAmounts.
	self addDepositReport.
	self addWithdrawReport.
	self addWithdrawByTransferReport.
	self addDepositByTransferReport.
	self addBalanceReport.
	^report contents.
	! !

!Report methodsFor: 'summary' stamp: 'MEV 6/2/2019 18:44:07'!
addBalanceReport
	
	report nextPutAll:'Balance = '.
	report nextPutAll:((self balance printString),'.').
	report newLine .! !

!Report methodsFor: 'summary' stamp: 'MEV 6/2/2019 18:17:44'!
addDepositByTransferReport

	report nextPutAll:'Entrada por transferencia de '.
	report nextPutAll:((depositByTransferAmount printString),'.').
	report newLine .
	! !

!Report methodsFor: 'summary' stamp: 'MEV 6/2/2019 18:17:50'!
addDepositReport

	report nextPutAll:'Depsito por '.
	report nextPutAll:((depositAmount printString),'.').
	report newLine .! !

!Report methodsFor: 'summary' stamp: 'MEV 6/2/2019 18:17:57'!
addWithdrawByTransferReport

	report nextPutAll:'Salida por transferencia de '.
	report nextPutAll:((withdrawByTransferAmount printString),'.').
	report newLine .
	! !

!Report methodsFor: 'summary' stamp: 'MEV 6/2/2019 18:18:03'!
addWithdrawReport
	
	report nextPutAll:'Extraccin por '.
	report nextPutAll:((withdrawAmount printString),'.').
	report newLine .! !


!Report methodsFor: 'initialization' stamp: 'MEV 6/2/2019 23:21:15'!
initializeFor:anAccount

	report := WriteStream on:(String new).
	account := anAccount.
	self setTransactionValuesToZero.
	self calculateTransactionAmounts.! !

!Report methodsFor: 'initialization' stamp: 'MEV 6/2/2019 18:20:22'!
setTransactionValuesToZero

	depositAmount := 0.
	withdrawAmount := 0.
	depositByTransferAmount := 0.
	withdrawByTransferAmount := 0.! !


!Report methodsFor: 'balance' stamp: 'MEV 6/2/2019 17:12:41'!
balance

	^(depositAmount + depositByTransferAmount - withdrawAmount - withdrawByTransferAmount) .
	! !


!Report methodsFor: 'transfer net' stamp: 'MEV 6/3/2019 10:34:53'!
transferNet

	self calculateTransactionAmounts.
	^depositByTransferAmount - withdrawByTransferAmount.
	! !


!Report methodsFor: 'transactions amounts' stamp: 'MEV 6/2/2019 23:21:36'!
amountsFor: aTransactionType

	|transactions|
	
	transactions := (account transactions) select:[:aTransaction | aTransaction isKindOf: aTransactionType].
	^transactions sum:[:aTransaction | aTransaction value.]  ifEmpty:[0] ! !

!Report methodsFor: 'transactions amounts' stamp: 'MEV 6/2/2019 23:21:36'!
calculateTransactionAmounts
		
		self setTransactionValuesToZero .
		depositAmount := self amountsFor: Deposit.
		withdrawAmount := self amountsFor:Withdraw.
		depositByTransferAmount := self amountsFor: DepositLeg.
		withdrawByTransferAmount := self amountsFor: WithdrawLeg.! !

!Report methodsFor: 'transactions amounts' stamp: 'MEV 6/2/2019 16:16:41'!
depositAmount

	^depositAmount .! !

!Report methodsFor: 'transactions amounts' stamp: 'MEV 6/2/2019 16:22:00'!
depositByTransferAmount

	^depositByTransferAmount .! !

!Report methodsFor: 'transactions amounts' stamp: 'MEV 6/2/2019 16:16:51'!
withdrawAmount

	^withdrawAmount .! !

!Report methodsFor: 'transactions amounts' stamp: 'MEV 6/2/2019 16:22:08'!
withdrawByTransferAmount

	^withdrawByTransferAmount .! !

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

!classDefinition: 'Report class' category: #PortfolioTreePrinter!
Report class
	instanceVariableNames: ''!

!Report class methodsFor: 'class initialization' stamp: 'MEV 6/2/2019 16:06:23'!
on:anAccount

	^self new initializeFor:anAccount.! !
