!classDefinition: #CashierTest category: #'TusLibros.com'!
TestCase subclass: #CashierTest
	instanceVariableNames: 'anotherShoppingCart testObjects debitBehavior'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'TusLibros.com'!

!CashierTest methodsFor: 'setup' stamp: 'MEV 6/24/2019 10:05:16'!
checkCreditCard: aCreditCard withTransactionAmount: anAmount

	^debitBehavior value: anAmount value: aCreditCard 
	
	! !

!CashierTest methodsFor: 'setup' stamp: 'MEV 6/24/2019 10:05:40'!
setUp

	testObjects := StoreTestObjects new.
	debitBehavior := [ :anAmount :aCreditCard | ]
	
	! !


!CashierTest methodsFor: 'tests' stamp: 'MEV 6/24/2019 15:33:17'!
test01CanNotCheckOutAnEmptyCart

	|aCashier shoppingCart aValidCreditCard|
	shoppingCart := testObjects createCart.
	aValidCreditCard := testObjects notExpiredCreditCard .
	
	
	aCashier := Cashier withCart: shoppingCart cc: aValidCreditCard andMP: self .
	
	self should:[aCashier checkOut.]
		raise: Error
		withExceptionDo: [ :anError | self assert: Cashier cartIsEmptyErrorSignal equals: anError messageText.
							self assert:(aCashier transactions isEmpty)].
	
		! !

!CashierTest methodsFor: 'tests' stamp: 'MEV 6/24/2019 10:06:35'!
test02ACheckedOutBookHasBeenSuccesfullyCheckedOut

	|aCashier shoppingCart aValidCreditCard|
	shoppingCart := testObjects createCart.
	aValidCreditCard := testObjects notExpiredCreditCard .

	shoppingCart add: 'ISBN1' withAmount: 2.
	
	aCashier := Cashier withCart: shoppingCart cc: aValidCreditCard andMP: self .
	
	aCashier checkOut .
	
	self assert:(aCashier hasCheckedOut:'ISBN1').
	
	
	
		! !

!CashierTest methodsFor: 'tests' stamp: 'MEV 6/24/2019 10:06:45'!
test03ACheckedOutBookHasBeenSuccesfullyCheckedOutWithItsCorrectQuantity

	|aCashier shoppingCart aValidCreditCard|
	shoppingCart := testObjects createCart.
	aValidCreditCard := testObjects notExpiredCreditCard .
	
	shoppingCart add: 'ISBN1' withAmount: 2.
	
	aCashier := Cashier withCart: shoppingCart cc: aValidCreditCard andMP: self .
	
	aCashier checkOut .
	
	self assert:(aCashier hasCheckedOut:'ISBN1').
	self assert:(aCashier checkedOutAmount:'ISBN1')equals:2.
	
	
	
		! !

!CashierTest methodsFor: 'tests' stamp: 'MEV 6/24/2019 15:33:28'!
test04CashierDoesNotCheckOutWhatIsNotInTheCart

	|aCashier shoppingCart aValidCreditCard|
	shoppingCart := testObjects createCart.
	aValidCreditCard := testObjects notExpiredCreditCard .
	
	shoppingCart add: 'ISBN1' withAmount: 2.
	
	aCashier := Cashier withCart: shoppingCart cc: aValidCreditCard andMP: self .
	
	aCashier checkOut .
	
	self assert:(aCashier transactions) equals: (shoppingCart cartContents ).
	
	
	
		! !

!CashierTest methodsFor: 'tests' stamp: 'MEV 6/24/2019 10:07:06'!
test05TotalAmountToCheckOutEqualsPriceOfCartContents

	|aCashier shoppingCart aValidCreditCard|
	shoppingCart := testObjects createCart.
	aValidCreditCard := testObjects notExpiredCreditCard .
	
	shoppingCart add: 'ISBN1' withAmount: 2.
	
	aCashier := Cashier withCart: shoppingCart cc: aValidCreditCard andMP: self .
	
	self assert:(aCashier totalAmount)equals:100.0.
	
	
	
		! !

!CashierTest methodsFor: 'tests' stamp: 'MEV 6/24/2019 10:10:03'!
test06TotalAmountOnlyHasTwoDecimalDigits

	|aCashier shoppingCart aValidCreditCard|
	shoppingCart := testObjects createAnotherCart.
	aValidCreditCard := testObjects notExpiredCreditCard .
	
	shoppingCart add: 'ISBN1' withAmount: 2.
	
	aCashier := Cashier withCart: shoppingCart cc: aValidCreditCard andMP: self .
	
	self assert:(aCashier totalAmount)equals:98.67.
	
	
	
		! !

!CashierTest methodsFor: 'tests' stamp: 'MEV 6/24/2019 15:33:38'!
test07MerchantProcessorDoesNotAcceptACreditCardWithAnIncorrectCreditCardNumber

	|aCashier shoppingCart anInvalidCreditCard|
	shoppingCart := testObjects createCart.
	anInvalidCreditCard := testObjects wrongNumberCreditCard .
	
	shoppingCart add: 'ISBN1' withAmount: 2.
	
	debitBehavior := [ :anAmount :aCreditCard | self error: Cashier incorrectCreditCardNumberErrorSignal].
	aCashier := Cashier withCart: shoppingCart cc: anInvalidCreditCard andMP: self .
	
	self should:[aCashier checkOut.]
		raise: Error
		withExceptionDo: [ :anError | self assert: Cashier incorrectCreditCardNumberErrorSignal equals: anError messageText.
							self assert:(aCashier transactions isEmpty)].
	
	
	
		! !

!CashierTest methodsFor: 'tests' stamp: 'MEV 6/24/2019 15:33:44'!
test08MerchantProcessorDoesNotAcceptACreditCardWithAnOwnerWithALongName

	|aCashier shoppingCart anInvalidCreditCard|
	shoppingCart := testObjects createCart.
	anInvalidCreditCard := testObjects wrongNameCreditCard .
	
	shoppingCart add: 'ISBN1' withAmount: 2.
	
	debitBehavior := [ :anAmount :aCreditCard | self error: Cashier ownerNameTooLongErrorSignal].
	
	aCashier := Cashier withCart: shoppingCart cc: anInvalidCreditCard andMP: self .
	
	self should:[aCashier checkOut.]
		raise: Error
		withExceptionDo: [ :anError | self assert: Cashier ownerNameTooLongErrorSignal equals: anError messageText.
							self assert:(aCashier transactions isEmpty)].
	
	
	
		! !

!CashierTest methodsFor: 'tests' stamp: 'MEV 6/24/2019 15:33:51'!
test09MerchantProcessorDoesNotAcceptATransactionForACreditCardWithoutAnyFunds

	|aCashier shoppingCart aValidCreditCard|
	shoppingCart := testObjects createCart.
	aValidCreditCard := testObjects notExpiredCreditCard .
	
	shoppingCart add: 'ISBN1' withAmount: 2.
	
	debitBehavior := [ :anAmount :aCreditCard | self error: Cashier noFundsInCreditCardErrorSignal].
	
	aCashier := Cashier withCart: shoppingCart cc: aValidCreditCard andMP: self .
	
	self should:[aCashier checkOut.]
		raise: Error
		withExceptionDo: [ :anError | self assert: Cashier noFundsInCreditCardErrorSignal equals: anError messageText.
							self assert:(aCashier transactions isEmpty)].
	
	
	
		! !

!CashierTest methodsFor: 'tests' stamp: 'MEV 6/24/2019 15:33:53'!
test10MerchantProcessorDoesNotAcceptAnExpiredCreditCard

	|aCashier shoppingCart anInvalidCreditCard|
	shoppingCart := testObjects createCart.
	anInvalidCreditCard := testObjects expiredCreditCard .
	
	shoppingCart add: 'ISBN1' withAmount: 2.
	
	debitBehavior := [ :anAmount :aCreditCard | self error: Cashier expiredCreditCardErrorSignal].
	
	aCashier := Cashier withCart: shoppingCart cc: anInvalidCreditCard andMP: self .
	
	self should:[aCashier checkOut.]
		raise: Error
		withExceptionDo: [ :anError | self assert: Cashier expiredCreditCardErrorSignal equals: anError messageText.
							self assert:(aCashier transactions isEmpty)].
	
	
	
		! !


!classDefinition: #RestAPITest category: #'TusLibros.com'!
TestCase subclass: #RestAPITest
	instanceVariableNames: 'logInBehavior newCatalog testObjects'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'TusLibros.com'!

!RestAPITest methodsFor: 'createCart tests' stamp: 'MEV 6/24/2019 11:05:35'!
test01CanNotCreateACartWithAnIncorrectClientIdOrAnIncorrectPassword

	|restAPI|
	
	restAPI := RestAPI new.
	
	logInBehavior := [ :clientId :clientPassword | self error: RestAPI invalidClientIDOrPasswordErrorMessage].
	
	self should: [restAPI createCartWithId: 'Invalid ID' andPassword: 'password' andValidateUsing:self]
		raise:Error
	  	withExceptionDo: 
					[ :anError | self assert: RestAPI invalidClientIDOrPasswordErrorMessage equals: anError messageText.
					self assert:(restAPI shoppingCarts isEmpty)].
				
	self should: [restAPI createCartWithId: 'aClientID' andPassword: 'an invalid password' andValidateUsing:self]
		raise:Error
	  	withExceptionDo: 
					[ :anError | self assert: RestAPI invalidClientIDOrPasswordErrorMessage equals: anError messageText.
					self assert:(restAPI shoppingCarts isEmpty)].	! !

!RestAPITest methodsFor: 'createCart tests' stamp: 'MEV 6/24/2019 13:24:20'!
test02CreatingANewCartReturnsItsID

	|restAPI|
	
	restAPI := RestAPI new.
	
	self assert:('0|1')equals:(restAPI createCartWithId: 'aClientID' andPassword: 'aPassword' andValidateUsing: self).
	self assert:((restAPI shoppingCartOfID: 1) isEmpty )! !

!RestAPITest methodsFor: 'createCart tests' stamp: 'MEV 6/24/2019 11:15:40'!
test03CreatingMoreThanOneCartGivesEachCartADifferentID

	|restAPI|
	
	restAPI := RestAPI new.
	
	self assert:('0|1')equals:(restAPI createCartWithId: 'aFirstClientID' andPassword: 'aFirstPassword' andValidateUsing: self).
	self assert:('0|2')equals:(restAPI createCartWithId: 'aSecondClientID' andPassword: 'aSecondPassword' andValidateUsing: self).
	self assert:('0|3')equals:(restAPI createCartWithId: 'aThirdClientID' andPassword: 'aThirdPassword' andValidateUsing: self).
	
	! !


!RestAPITest methodsFor: 'behavior' stamp: 'MEV 6/24/2019 10:43:51'!
logInWithId: clientId andPassword: clientPassword

	^logInBehavior value: clientId value: clientPassword ! !


!RestAPITest methodsFor: 'addToCart tests' stamp: 'MEV 6/24/2019 11:22:00'!
test04CanNotAddBooksToAnInvalidCart

	|restAPI|
	
	restAPI := RestAPI new.
	
	self should: [restAPI addToCart: 1 bookISBN: 'ISBN1' bookQuantity: 2]
		raise:Error
	  	withExceptionDo: 
					[ :anError | self assert: RestAPI invalidCartErrorMessage equals: anError messageText.
					self assert:(restAPI shoppingCarts isEmpty)].
	
	! !

!RestAPITest methodsFor: 'addToCart tests' stamp: 'MEV 6/24/2019 11:41:50'!
test05CanNotAddBooksThatAreNotInTheCatalog

	|restAPI|
	
	restAPI := RestAPI new.
	restAPI createCartWithId: 'aFirstClientID' andPassword: 'aFirstPassword' andValidateUsing: self.
	
	self should: [restAPI addToCart: 1 bookISBN: 'ISBN2' bookQuantity: 2]
		raise:Error
	  	withExceptionDo: 
					[ :anError | self assert: RestAPI invalidBookErrorMessage equals: anError messageText.
					self assert:((restAPI shoppingCartOfID:1) isEmpty)].
	
	! !

!RestAPITest methodsFor: 'addToCart tests' stamp: 'MEV 6/24/2019 11:41:22'!
test06CanNotAddAnInvalidAmountOfBooksToTheCart

	|restAPI|
	
	restAPI := RestAPI new.
	restAPI createCartWithId: 'aFirstClientID' andPassword: 'aFirstPassword' andValidateUsing: self.
	
	self should: [restAPI addToCart: 1 bookISBN: 'ISBN1' bookQuantity: 0]
		raise:Error
	  	withExceptionDo: 
					[ :anError | self assert: RestAPI invalidAmountOfBooksErrorMessage equals: anError messageText.
					self assert:((restAPI shoppingCartOfID:1) isEmpty)].
	
	! !

!RestAPITest methodsFor: 'addToCart tests' stamp: 'MEV 6/24/2019 11:52:02'!
test07AddingABookToACartAddsItWithitsCorrectQuantityAndAddsNothingElse

	|restAPI shoppingCart|
	
	restAPI := RestAPI new.
	restAPI createCartWithId: 'aFirstClientID' andPassword: 'aFirstPassword' andValidateUsing: self.
	
	self assert:('0|OK') equals:(restAPI addToCart: 1 bookISBN: 'ISBN1' bookQuantity: 2).
	
	shoppingCart := restAPI shoppingCartOfID: 1.
	
	self assert:(shoppingCart cartContents size)equals:1.
	self assert:(shoppingCart hasInCart: 'ISBN1').
	self assert:(shoppingCart amountOf: 'ISBN1')equals:2.
	
	
	
	
	! !

!RestAPITest methodsFor: 'addToCart tests' stamp: 'MEV 6/24/2019 11:50:16'!
test08AddingABookToACartOnlyAddsItToThatCart

	|restAPI shoppingCart anotherShoppingCart|
	
	restAPI := RestAPI new.
	restAPI createCartWithId: 'aFirstClientID' andPassword: 'aFirstPassword' andValidateUsing: self.
	restAPI createCartWithId: 'aSecondClientID' andPassword: 'aSecondPassword' andValidateUsing: self.
	restAPI addToCart: 1 bookISBN: 'ISBN1' bookQuantity: 2.
	shoppingCart := restAPI shoppingCartOfID: 1.
	anotherShoppingCart := restAPI shoppingCartOfID: 2.
	
	self deny:(shoppingCart isEmpty ).
	self assert:(anotherShoppingCart isEmpty )
	
	
	
	
	! !


!RestAPITest methodsFor: 'setUp/tearDown' stamp: 'MEV 6/24/2019 12:40:12'!
setUp

	logInBehavior := [ :clientId :clientPassword | ].
	newCatalog := Dictionary new at: 'ISBN1' put:50; at: 'ISBN2' put:30; yourself.
	testObjects := StoreTestObjects new.! !


!RestAPITest methodsFor: 'checkOutCart tests' stamp: 'MEV 6/24/2019 14:12:40'!
test12CanCheckOutAValidCart

	|restAPI aValidCreditCard|
	
	restAPI := RestAPI withMP:testObjects .
	aValidCreditCard := testObjects notExpiredCreditCard .
	
	restAPI createCartWithId: 'aFirstClientID' andPassword: 'aFirstPassword' andValidateUsing: self.
	restAPI addToCart: 1 bookISBN: 'ISBN1' bookQuantity: 2.
	
	self assert:(restAPI checkOutCart:1 ccn:(aValidCreditCard ccn) cced:(aValidCreditCard cced) cco:(aValidCreditCard cco))equals:'0|1'.
	
	
	
	
	! !

!RestAPITest methodsFor: 'checkOutCart tests' stamp: 'MEV 6/24/2019 12:49:37'!
test13CanNotCheckOutAnInvalidCart

	|restAPI|

	
	restAPI := RestAPI withMP:testObjects .
	
	self should: [restAPI checkOutCart:1 ccn:'1234123412341234' cced:(testObjects today) cco: 'Juan']
		raise:Error
	  	withExceptionDo: 
					[ :anError | self assert: RestAPI invalidCartErrorMessage equals: anError messageText.].
	
	
	
	
	! !

!RestAPITest methodsFor: 'checkOutCart tests' stamp: 'MEV 6/24/2019 14:01:02'!
test14CanNotCheckOutAnEmptyCart

	|restAPI validCreditCard|

	
	restAPI := RestAPI withMP:testObjects .
	validCreditCard := testObjects notExpiredCreditCard .
	restAPI createCartWithId: 'aFirstClientID' andPassword: 'aFirstPassword' andValidateUsing: self.
	
	self should: [restAPI checkOutCart:1 ccn: (validCreditCard ccn) cced:(validCreditCard cced ) cco: (validCreditCard cco)]
		raise:Error
	  	withExceptionDo: 
					[ :anError | self assert: RestAPI emptyCartErrorMessage equals: anError messageText.].
	
	
	
	
	! !

!RestAPITest methodsFor: 'checkOutCart tests' stamp: 'MEV 6/24/2019 14:02:15'!
test15CanNotCheckOutACartWithACreditCardWithAWrongNumber

	|restAPI invalidCreditCard|

	testObjects debitBehavior: [ :anAmount :aCreditCard | self error: RestAPI creditCardWithWrongNumberErrorSignal].
	invalidCreditCard := testObjects wrongNumberCreditCard .
	restAPI := RestAPI withMP:testObjects .
	
	restAPI createCartWithId: 'aFirstClientID' andPassword: 'aFirstPassword' andValidateUsing: self.
	restAPI addToCart: 1 bookISBN: 'ISBN1' bookQuantity: 2.
	
	
	self should: [restAPI checkOutCart:1 ccn: (invalidCreditCard ccn) cced:(invalidCreditCard cced ) cco: (invalidCreditCard cco)]
		raise:Error
	  	withExceptionDo: 
					[ :anError | self assert: RestAPI creditCardWithWrongNumberErrorSignal equals: anError messageText.].
	
	
	
	
	! !

!RestAPITest methodsFor: 'checkOutCart tests' stamp: 'MEV 6/24/2019 14:03:05'!
test16CanNotCheckOutACartWithACreditCardWithAnInvalidName

	|restAPI invalidCreditCard|

	testObjects debitBehavior: [ :anAmount :aCreditCard | self error: RestAPI creditCardNameTooLongErrorSignal].
	invalidCreditCard := testObjects wrongNameCreditCard .
	restAPI := RestAPI withMP:testObjects .
	
	restAPI createCartWithId: 'aFirstClientID' andPassword: 'aFirstPassword' andValidateUsing: self.
	restAPI addToCart: 1 bookISBN: 'ISBN1' bookQuantity: 2.
	
	
	self should: [restAPI checkOutCart:1 ccn:(invalidCreditCard ccn) cced:(invalidCreditCard cced ) cco: (invalidCreditCard cco)]
		raise:Error
	  	withExceptionDo: 
					[ :anError | self assert: RestAPI creditCardNameTooLongErrorSignal equals: anError messageText.].
	
	
	
	
	! !

!RestAPITest methodsFor: 'checkOutCart tests' stamp: 'MEV 6/24/2019 14:03:43'!
test17CanNotCheckOutACartWithAnExpiredCreditCard

	|restAPI invalidCreditCard|

	testObjects debitBehavior: [ :anAmount :aCreditCard | self error: RestAPI expiredCreditCardErrorSignal].
	invalidCreditCard := testObjects expiredCreditCard .
	restAPI := RestAPI withMP:testObjects .
	
	restAPI createCartWithId: 'aFirstClientID' andPassword: 'aFirstPassword' andValidateUsing: self.
	restAPI addToCart: 1 bookISBN: 'ISBN1' bookQuantity: 2.
	
	
	self should: [restAPI checkOutCart:1 ccn:(invalidCreditCard ccn) cced:(invalidCreditCard cced ) cco: (invalidCreditCard cco)]
		raise:Error
	  	withExceptionDo: 
					[ :anError | self assert: RestAPI expiredCreditCardErrorSignal equals: anError messageText.].
	
	
	
	
	! !

!RestAPITest methodsFor: 'checkOutCart tests' stamp: 'MEV 6/24/2019 15:36:12'!
test18CheckingOutAValidCartAddsItsContentsToTransactions

	|restAPI aValidCreditCard|
	
	restAPI := RestAPI withMP:testObjects .
	aValidCreditCard := testObjects notExpiredCreditCard .
	
	restAPI createCartWithId: 'aFirstClientID' andPassword: 'aFirstPassword' andValidateUsing: self.
	restAPI addToCart: 1 bookISBN: 'ISBN1' bookQuantity: 2.
	
	self assert:(restAPI checkOutCart:1 ccn:(aValidCreditCard ccn) cced:(aValidCreditCard cced) cco:(aValidCreditCard cco))equals:'0|1'.
	self assert:((restAPI transactionOfID:1) transactions) equals: ((restAPI shoppingCartOfID:1) cartContents ).
	
	
	
	
	! !


!RestAPITest methodsFor: 'listCart tests' stamp: 'MEV 6/24/2019 12:04:49'!
test09CanNotListAnInvalidCart

	|restAPI|
	
	restAPI := RestAPI new.
	
	self should: [restAPI listCart: 1]
		raise:Error
	  	withExceptionDo: 
					[ :anError | self assert: RestAPI invalidCartErrorMessage equals: anError messageText.].
	
	
	
	
	! !

!RestAPITest methodsFor: 'listCart tests' stamp: 'MEV 6/24/2019 12:04:35'!
test10CanNotListAnEmptyCart

	|restAPI|
	
	restAPI := RestAPI new.
	restAPI createCartWithId: 'aFirstClientID' andPassword: 'aFirstPassword' andValidateUsing: self.
	
	self should: [restAPI listCart: 1]
		raise:Error
	  	withExceptionDo: 
					[ :anError | self assert: RestAPI emptyCartErrorMessage equals: anError messageText.].
	
	
	
	
	! !

!RestAPITest methodsFor: 'listCart tests' stamp: 'MEV 6/24/2019 12:11:40'!
test11CanListAValidCart

	|restAPI|
	
	restAPI := RestAPI new.
	restAPI changeCatalogTo: newCatalog .
	restAPI createCartWithId: 'aFirstClientID' andPassword: 'aFirstPassword' andValidateUsing: self.
	restAPI addToCart: 1 bookISBN: 'ISBN1' bookQuantity: 2.
	restAPI addToCart: 1 bookISBN: 'ISBN2' bookQuantity: 4.
	
	self assert:(restAPI listCart: 1)equals:'0|ISBN1|2|ISBN2|4'
	
	
	
	
	! !


!RestAPITest methodsFor: 'listPurchases tests' stamp: 'MEV 6/24/2019 14:27:00'!
test19CanNotListPurchasesOfAnInvalidClient

	|restAPI|
	
	restAPI := RestAPI new.
	logInBehavior := [ :clientId :clientPassword | self error: RestAPI invalidClientIDOrPasswordErrorMessage].
	
	self should: [restAPI listPurchasesWithId: 'anInvalidClientID' andPassword: 'aValidPassword' andValidateUsing: self.]
		raise:Error
	  	withExceptionDo: 
					[ :anError | self assert: RestAPI invalidClientIDOrPasswordErrorMessage equals: anError messageText.].
	
	
	
	
	! !

!RestAPITest methodsFor: 'listPurchases tests' stamp: 'MEV 6/24/2019 14:28:42'!
test20CanNotListPurchasesOfAClientWithAnInvalidPassword

	|restAPI|
	
	restAPI := RestAPI new.
	logInBehavior := [ :clientId :clientPassword | self error: RestAPI invalidClientIDOrPasswordErrorMessage].
	
	self should: [restAPI listPurchasesWithId: 'aValidClientID' andPassword: 'anInvalidPassword' andValidateUsing: self.]
		raise:Error
	  	withExceptionDo: 
					[ :anError | self assert: RestAPI invalidClientIDOrPasswordErrorMessage equals: anError messageText.].
	
	
	
	
	! !

!RestAPITest methodsFor: 'listPurchases tests' stamp: 'MEV 6/24/2019 14:50:05'!
test21CanNotListPurchasesOfAClientWithNoPurchases

	|restAPI|
	
	restAPI := RestAPI new.
	
	self should: [restAPI listPurchasesWithId: 'aValidClientID' andPassword: 'aValidPassword' andValidateUsing: self.]
		raise:Error
	  	withExceptionDo: 
					[ :anError | self assert: RestAPI noPurchasesErrorMessage equals: anError messageText.].
	
	
	
	
	! !

!RestAPITest methodsFor: 'listPurchases tests' stamp: 'MEV 6/24/2019 15:11:06'!
test22CanListPurchasesOfAValidClient

	|restAPI aValidCreditCard|
	
	
	restAPI := RestAPI withMP: testObjects.
	aValidCreditCard := testObjects notExpiredCreditCard .
	
	restAPI changeCatalogTo: newCatalog .
	restAPI createCartWithId: 'aFirstClientID' andPassword: 'aFirstPassword' andValidateUsing: self.
	restAPI addToCart: 1 bookISBN: 'ISBN1' bookQuantity: 2.
	restAPI addToCart: 1 bookISBN: 'ISBN2' bookQuantity: 4.
	
	restAPI checkOutCart:1 ccn:(aValidCreditCard ccn) cced:(aValidCreditCard cced) cco:(aValidCreditCard cco).
	
	self assert:(restAPI listPurchasesWithId: 'aFirstClientID' andPassword:  'aFirstPassword' andValidateUsing: self)
		equals: ('0|ISBN1|2|ISBN2|4|220.0')
	
	
	
	
	! !

!RestAPITest methodsFor: 'listPurchases tests' stamp: 'MEV 6/24/2019 15:40:38'!
test23CanListPurchasesOfAValidClientThatCheckedOutMoreThanOneCart

	|restAPI aValidCreditCard|
	
	
	restAPI := RestAPI withMP: testObjects.
	aValidCreditCard := testObjects notExpiredCreditCard .
	
	restAPI changeCatalogTo: newCatalog .
	restAPI createCartWithId: 'aFirstClientID' andPassword: 'aFirstPassword' andValidateUsing: self.
	restAPI addToCart: 1 bookISBN: 'ISBN1' bookQuantity: 2.
	restAPI addToCart: 1 bookISBN: 'ISBN2' bookQuantity: 4.
	restAPI checkOutCart:1 ccn:(aValidCreditCard ccn) cced:(aValidCreditCard cced) cco:(aValidCreditCard cco).
	
	restAPI createCartWithId: 'aFirstClientID' andPassword: 'aFirstPassword' andValidateUsing: self.
	restAPI addToCart: 2 bookISBN: 'ISBN1' bookQuantity: 3.
	restAPI addToCart: 2 bookISBN: 'ISBN2' bookQuantity: 6.
	restAPI checkOutCart:2 ccn:(aValidCreditCard ccn) cced:(aValidCreditCard cced) cco:(aValidCreditCard cco).
	
	self assert:(restAPI listPurchasesWithId: 'aFirstClientID' andPassword:  'aFirstPassword' andValidateUsing: self)
		equals: ('0|ISBN1|2|ISBN2|4|ISBN1|3|ISBN2|6|550.0')
	
	
	
	
	! !


!RestAPITest methodsFor: 'expired cart tests' stamp: 'MEV 6/24/2019 16:16:43'!
test24CanNotAddBooksToAnExpiredCartAndTheCartIsRemoved

	|restAPI|
	
	
	restAPI := RestAPI new.
	restAPI createCartWithId: 'aFirstClientID' andPassword: 'aFirstPassword' andValidateUsing: self.
	
	restAPI cart: 1 lastAccessedAt:(DateAndTime now subtractDuration: (31 minutes)).
	
	self should: [restAPI addToCart: 1 bookISBN: 'ISBN1' bookQuantity: 2.]
		raise:Error
	  	withExceptionDo: 
					[ :anError | self assert: RestAPI expiredCartErrorMessage equals: anError messageText.].
	self should: [restAPI addToCart: 1 bookISBN: 'ISBN1' bookQuantity: 2.]
		raise:Error
	  	withExceptionDo: 
					[ :anError | self assert: RestAPI invalidCartErrorMessage equals: anError messageText.].
	! !

!RestAPITest methodsFor: 'expired cart tests' stamp: 'MEV 6/24/2019 16:20:16'!
test25CanNotListAnExpiredCartAndTheCartIsRemoved

	|restAPI|
	
	
	restAPI := RestAPI new.
	restAPI createCartWithId: 'aFirstClientID' andPassword: 'aFirstPassword' andValidateUsing: self.
	restAPI addToCart: 1 bookISBN: 'ISBN1' bookQuantity: 2.
	
	restAPI cart: 1 lastAccessedAt:(DateAndTime now subtractDuration: (31 minutes)).
	
	self should: [restAPI listCart: 1.]
		raise:Error
	  	withExceptionDo: 
					[ :anError | self assert: RestAPI expiredCartErrorMessage equals: anError messageText.].
	self should: [restAPI listCart: 1.]
		raise:Error
	  	withExceptionDo: 
					[ :anError | self assert: RestAPI invalidCartErrorMessage equals: anError messageText.].
	! !

!RestAPITest methodsFor: 'expired cart tests' stamp: 'MEV 6/24/2019 16:24:40'!
test26CanNotCheckOutAnExpiredCartAndTheCartIsRemoved

	|restAPI aValidCreditCard|
	
	
	restAPI := RestAPI withMP:testObjects .
	aValidCreditCard := testObjects notExpiredCreditCard .
	restAPI createCartWithId: 'aFirstClientID' andPassword: 'aFirstPassword' andValidateUsing: self.
	restAPI addToCart: 1 bookISBN: 'ISBN1' bookQuantity: 2.
	
	restAPI cart: 1 lastAccessedAt:(DateAndTime now subtractDuration: (31 minutes)).
	
	self should: [restAPI checkOutCart:1 ccn:(aValidCreditCard ccn) cced:(aValidCreditCard cced) cco:(aValidCreditCard cco)]
		raise:Error
	  	withExceptionDo: 
					[ :anError | self assert: RestAPI expiredCartErrorMessage equals: anError messageText.].
	self should: [restAPI checkOutCart:1 ccn:(aValidCreditCard ccn) cced:(aValidCreditCard cced) cco:(aValidCreditCard cco).]
		raise:Error
	  	withExceptionDo: 
					[ :anError | self assert: RestAPI invalidCartErrorMessage equals: anError messageText.].
	! !


!classDefinition: #ShoppingCartTest category: #'TusLibros.com'!
TestCase subclass: #ShoppingCartTest
	instanceVariableNames: 'testObjects'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'TusLibros.com'!

!ShoppingCartTest methodsFor: 'tests' stamp: 'MEV 6/24/2019 09:54:34'!
setUp

	testObjects := StoreTestObjects new.
	! !

!ShoppingCartTest methodsFor: 'tests' stamp: 'MEV 6/24/2019 09:55:17'!
test01AShoppingCartIsCreatedEmpty

	|shoppingCart|
	
	shoppingCart := testObjects createCart .
	self assert:(shoppingCart isEmpty).! !

!ShoppingCartTest methodsFor: 'tests' stamp: 'MEV 6/24/2019 09:55:29'!
test02AddingBooksToTheShoppingCartMakesItNotEmpty

	|shoppingCart|
	
	shoppingCart := testObjects createCart .
	
	shoppingCart add:'ISBN1' withAmount:2.
	
	self deny:(shoppingCart isEmpty).! !

!ShoppingCartTest methodsFor: 'tests' stamp: 'MEV 6/24/2019 09:55:35'!
test03CanNotAddBooksThatAreNotFromTheCatalog

	|shoppingCart|
	
	shoppingCart := testObjects createCart .
	
	self should:[shoppingCart add:'ISBN3' withAmount:2.]
		raise: Error
		withExceptionDo: [ :anError | self assert: ShoppingCart canNotAddBooksThatAreNotFromTheCatalogErrorSignal equals: anError messageText.
							self assert:(shoppingCart isEmpty)].
	
	
	
	! !

!ShoppingCartTest methodsFor: 'tests' stamp: 'MEV 6/24/2019 09:55:40'!
test04CanNotBuyZeroOrLessBooks

	
	|shoppingCart|
	
	shoppingCart := testObjects createCart .
	
	self should:[shoppingCart add:'ISBN1' withAmount:0.]
		raise: Error
		withExceptionDo: [ :anError | self assert: ShoppingCart invalidAmountOfBooksErrorSignal equals: anError messageText.
							self assert:(shoppingCart isEmpty)].
						
	self should:[shoppingCart add:'ISBN1' withAmount:(-1).]
		raise: Error
		withExceptionDo: [ :anError | self assert: ShoppingCart invalidAmountOfBooksErrorSignal equals: anError messageText.
							self assert:(shoppingCart isEmpty)].
	
	
	
	! !

!ShoppingCartTest methodsFor: 'tests' stamp: 'MEV 6/24/2019 09:55:46'!
test05WhatIBuyEndsUpInTheCart

	|shoppingCart|
	
	shoppingCart := testObjects createCart .
	
	shoppingCart add:'ISBN1' withAmount:2.
	
	self deny:(shoppingCart isEmpty).
	self assert:(shoppingCart hasInCart:'ISBN1').
	self assert:(shoppingCart amountOf:'ISBN1')equals:2.
	! !

!ShoppingCartTest methodsFor: 'tests' stamp: 'MEV 6/24/2019 09:55:50'!
test06TheCartDoesNotHaveBooksThatIDidNotBuy

	|shoppingCart|
	
	shoppingCart := testObjects createCart .

	shoppingCart add:'ISBN1' withAmount:2.
	
	self assert:(shoppingCart amountOf:'ISBN1')equals:2.
	self assert:(shoppingCart amountOfBooksInCart)equals:2.
	! !


!classDefinition: #Cashier category: #'TusLibros.com'!
Object subclass: #Cashier
	instanceVariableNames: 'shoppingCart transactions creditCard merchantProcessor'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'TusLibros.com'!

!Cashier methodsFor: 'initialize' stamp: 'MEV 6/10/2019 13:49:33'!
initializeWithCart: aShoppingCart cc: aCreditCard andMP: aMerchantProcessor .

	shoppingCart := aShoppingCart.
	creditCard:= aCreditCard .
	merchantProcessor := aMerchantProcessor .
	transactions := OrderedCollection new.! !


!Cashier methodsFor: 'check out' stamp: 'MEV 6/24/2019 10:29:08'!
checkOut

	(shoppingCart isEmpty )ifTrue: [self error: Cashier cartIsEmptyErrorSignal].
	
	merchantProcessor checkCreditCard:creditCard withTransactionAmount:(self totalAmount ) .
	self addContents: shoppingCart cartContents .
	^self totalAmount .! !


!Cashier methodsFor: 'adding contents' stamp: 'MEV 6/10/2019 09:45:39'!
addContents: aShoppingCartContents
	
	aShoppingCartContents do:[:aBookAndAmount | transactions add: aBookAndAmount ].! !


!Cashier methodsFor: 'transactions' stamp: 'MEV 6/24/2019 15:32:50'!
transactions

	^transactions.! !


!Cashier methodsFor: 'testing' stamp: 'MEV 6/24/2019 15:32:51'!
checkedOutAmount: anISBN

	|book|
	book := self transactions detect:[:aTransaction | aTransaction key = anISBN].
	^book value.! !

!Cashier methodsFor: 'testing' stamp: 'MEV 6/24/2019 15:32:51'!
hasCheckedOut: anISBN

	^self transactions anySatisfy: [:aTransaction | aTransaction key = anISBN].! !

!Cashier methodsFor: 'testing' stamp: 'MEV 6/24/2019 11:35:02'!
totalAmount

	|total stream|
	
	total := (shoppingCart cartContents )sum:[:aTransaction | |bookValue bookAmount|
									bookValue := (shoppingCart catalog)at:(aTransaction key).
									bookAmount := (aTransaction value).
									(bookValue *bookAmount ) ].
	stream := WriteStream on:String new.
	total printOn: stream fractionDigits: 2.	
	^stream contents asNumber.! !

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

!classDefinition: 'Cashier class' category: #'TusLibros.com'!
Cashier class
	instanceVariableNames: ''!

!Cashier class methodsFor: 'error signal' stamp: 'MEV 6/10/2019 09:22:26'!
cartIsEmptyErrorSignal

	^'Cart is empty'.! !

!Cashier class methodsFor: 'error signal' stamp: 'MEV 6/24/2019 10:13:30'!
expiredCreditCardErrorSignal

	^'Credit card has expired'.! !

!Cashier class methodsFor: 'error signal' stamp: 'MEV 6/24/2019 10:12:10'!
incorrectCreditCardNumberErrorSignal

	^'The credit card number is incorrect'.! !

!Cashier class methodsFor: 'error signal' stamp: 'MEV 6/24/2019 10:13:56'!
noFundsInCreditCardErrorSignal

	^'The credit card has no funds'.! !

!Cashier class methodsFor: 'error signal' stamp: 'MEV 6/24/2019 10:14:05'!
ownerNameTooLongErrorSignal

	^'The owner of the credit card has a name that is too long'.! !


!Cashier class methodsFor: 'initialization' stamp: 'MEV 6/10/2019 09:17:53'!
withCart: aShoppingCart cc: aCreditCard andMP: aMerchantProcessor .

	^self new initializeWithCart: aShoppingCart cc: aCreditCard andMP: aMerchantProcessor .! !


!classDefinition: #CreditCard category: #'TusLibros.com'!
Object subclass: #CreditCard
	instanceVariableNames: 'ccn cced cco'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'TusLibros.com'!

!CreditCard methodsFor: 'initialize' stamp: 'MEV 6/10/2019 13:22:24'!
withccn: aCreditCardNumber cced: aMonthOfYear cco: anOwner.

	ccn := aCreditCardNumber .
	cced:= aMonthOfYear .
	cco:= anOwner .! !


!CreditCard methodsFor: 'information' stamp: 'MEV 6/10/2019 09:14:14'!
cced

	^cced.! !

!CreditCard methodsFor: 'information' stamp: 'MEV 6/10/2019 09:14:08'!
ccn

	^ccn.! !

!CreditCard methodsFor: 'information' stamp: 'MEV 6/10/2019 09:14:23'!
cco

	^cco.! !

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

!classDefinition: 'CreditCard class' category: #'TusLibros.com'!
CreditCard class
	instanceVariableNames: ''!

!CreditCard class methodsFor: 'class initialization' stamp: 'MEV 6/10/2019 13:22:30'!
ccn: aCreditCardNumber cced: aMonthOfYear cco: anOwner

	^self new withccn: aCreditCardNumber cced: aMonthOfYear cco: anOwner.
	! !


!classDefinition: #RestAPI category: #'TusLibros.com'!
Object subclass: #RestAPI
	instanceVariableNames: 'shoppingCarts nextID catalog merchantProcessor transactionID logInBehavior clientsTransactions clientsCarts cashiers lastTimeAccessed'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'TusLibros.com'!

!RestAPI methodsFor: 'creating cart' stamp: 'MEV 6/24/2019 15:48:08'!
createCartWithId: aClientId andPassword: aPassword andValidateUsing: idAndPasswordDB
	
	|shoppingCart cartID|
	idAndPasswordDB logInWithId: aClientId andPassword: aPassword.
	
	shoppingCart := ShoppingCart createCartwithCatalog: catalog.
	cartID := nextID .
	shoppingCarts at: cartID put: shoppingCart.
	clientsCarts at:cartID put:aClientId.
	self cart: cartID lastAccessedAt: (DateAndTime now).
	(clientsTransactions includesKey: aClientId)ifFalse:[clientsTransactions at:aClientId put:OrderedCollection new].
	nextID := nextID+1.
	^'0|',cartID printString.! !


!RestAPI methodsFor: 'accessing' stamp: 'MEV 6/24/2019 15:49:11'!
cart: aCartID lastAccessedAt: aDateAndTime

	lastTimeAccessed at:aCartID put:aDateAndTime .! !

!RestAPI methodsFor: 'accessing' stamp: 'MEV 6/24/2019 15:31:44'!
cashiers

	^cashiers .! !

!RestAPI methodsFor: 'accessing' stamp: 'MEV 6/24/2019 14:46:40'!
clientOfCart: aCartID

	^clientsCarts at:aCartID.! !

!RestAPI methodsFor: 'accessing' stamp: 'MEV 7/11/2019 15:08:36'!
merchantProcessor

	^merchantProcessor .! !

!RestAPI methodsFor: 'accessing' stamp: 'MEV 7/10/2019 20:27:22'!
nextID

	^nextID .! !

!RestAPI methodsFor: 'accessing' stamp: 'MEV 6/24/2019 11:14:03'!
shoppingCartOfID: aCartID

	^shoppingCarts at:aCartID .! !

!RestAPI methodsFor: 'accessing' stamp: 'MEV 6/24/2019 10:54:58'!
shoppingCarts

	^shoppingCarts .! !

!RestAPI methodsFor: 'accessing' stamp: 'MEV 6/24/2019 15:30:36'!
transactionOfID: aTransactionID

	^cashiers at:aTransactionID .! !

!RestAPI methodsFor: 'accessing' stamp: 'MEV 6/24/2019 14:49:20'!
transactionsOfClient: aClientID

	^clientsTransactions at:aClientID ifAbsent:[self error: RestAPI noPurchasesErrorMessage].! !


!RestAPI methodsFor: 'testing' stamp: 'MEV 6/24/2019 11:47:36'!
validateBook: aBookISBN ofCart: aShoppingCart 

	(aShoppingCart catalog includesKey: aBookISBN)ifFalse:[self error: RestAPI invalidBookErrorMessage].! !

!RestAPI methodsFor: 'testing' stamp: 'MEV 6/24/2019 12:20:21'!
validateCartID: aCartID

	^shoppingCarts at: aCartID ifAbsent:[self error: RestAPI invalidCartErrorMessage].! !

!RestAPI methodsFor: 'testing' stamp: 'MEV 6/24/2019 16:18:17'!
validateExpirationTimeOfCart: aCartID

	((DateAndTime now subtractDateAndtime: (lastTimeAccessed at:aCartID )) >= (30 minutes))ifTrue: 
						[shoppingCarts removeKey: aCartID .
							self error: RestAPI expiredCartErrorMessage ].! !

!RestAPI methodsFor: 'testing' stamp: 'MEV 6/24/2019 11:48:11'!
validateQuantity: aBookQuantity 

	(aBookQuantity strictlyPositive )ifFalse:[self error: RestAPI invalidAmountOfBooksErrorMessage ].! !


!RestAPI methodsFor: 'catalog' stamp: 'MEV 7/10/2019 20:50:39'!
catalog

	^catalog.! !

!RestAPI methodsFor: 'catalog' stamp: 'MEV 6/24/2019 12:09:44'!
changeCatalogTo: aNewCatalog

	catalog:= aNewCatalog .
! !

!RestAPI methodsFor: 'catalog' stamp: 'MEV 7/11/2019 13:54:35'!
defaultCatalog

	^Dictionary new  at: 'ISBN1' put:50; 
						at: 'ISBN2' put:50; 
						at: 'ISBN3' put:50; 
						at: 'ISBN4' put:50; 
						at: 'ISBN5' put:50; 
						at: 'ISBN6' put:50; 
						at: 'ISBN7' put:50; 
						yourself.! !


!RestAPI methodsFor: 'listing cart' stamp: 'MEV 6/24/2019 16:22:02'!
listCart: aCartID 

	|shoppingCart list|
	
	shoppingCart := self validateCartID: aCartID.
	self validateExpirationTimeOfCart: aCartID .
	(shoppingCart isEmpty )ifTrue: [self error: RestAPI emptyCartErrorMessage ].
	
	list := '0'.
	(shoppingCart cartContents )do:[:aBookAndAnAmount | list := list,(String streamContents: [:str | str 
																								nextPutAll: '|';
																								nextPutAll: (aBookAndAnAmount key);
																								nextPutAll: '|';
																								nextPutAll: (aBookAndAnAmount value printString )])].
	self cart: aCartID lastAccessedAt: (DateAndTime now).
	^list.
! !


!RestAPI methodsFor: 'adding to cart' stamp: 'MEV 6/24/2019 16:21:41'!
addToCart: aCartID bookISBN: aBookISBN bookQuantity: aBookQuantity

	|shoppingCart|
	
	shoppingCart := self validateCartID: aCartID .
	self validateExpirationTimeOfCart: aCartID.
	self validateBook: aBookISBN ofCart:shoppingCart .
	self validateQuantity: aBookQuantity .
	
	shoppingCart add: aBookISBN withAmount: aBookQuantity .
	self cart: aCartID lastAccessedAt: (DateAndTime now).
	^'0|OK'.! !


!RestAPI methodsFor: 'initialization' stamp: 'MEV 6/24/2019 15:46:46'!
initialize

	shoppingCarts := Dictionary new.
	cashiers := Dictionary new.
	clientsTransactions := Dictionary new.
	clientsCarts := Dictionary new.
	lastTimeAccessed := Dictionary new.
	nextID := 1.
	transactionID := 1.
	catalog := self defaultCatalog .! !

!RestAPI methodsFor: 'initialization' stamp: 'MEV 6/24/2019 15:46:51'!
initializeWithMerchantProcessor: aMerchantProcessor 

	shoppingCarts := Dictionary new.
	cashiers := Dictionary new.
	clientsTransactions := Dictionary new.
	clientsCarts := Dictionary new.
	lastTimeAccessed := Dictionary new.
	nextID := 1.
	transactionID := 1.
	catalog :=self defaultCatalog .
	merchantProcessor := aMerchantProcessor .! !


!RestAPI methodsFor: 'checking out cart' stamp: 'MEV 6/24/2019 16:26:02'!
checkOutCart:aCartID ccn: aCCNumber cced: aCCExpirationDate cco: aCCOwner

	|shoppingCart creditCard aCashier aTransactionID aClientTransactions|
	
	shoppingCart := self validateCartID: aCartID.
	self validateExpirationTimeOfCart: aCartID.
	(shoppingCart isEmpty )ifTrue: [self error: RestAPI emptyCartErrorMessage ].
	
	creditCard := CreditCard ccn: aCCNumber cced: aCCExpirationDate cco: aCCOwner.
	aCashier := Cashier withCart: shoppingCart cc: creditCard andMP: merchantProcessor .
	aCashier checkOut .
	
	aTransactionID := transactionID.
	cashiers at:aTransactionID put:aCashier.
	
	aClientTransactions := clientsTransactions at: (self clientOfCart: aCartID) .
	aClientTransactions add: aTransactionID .
	transactionID  := transactionID +1.
	
	self cart: aCartID lastAccessedAt: (DateAndTime now).
	
	^'0|',aTransactionID printString .! !


!RestAPI methodsFor: 'listing purchases tests' stamp: 'MEV 6/24/2019 15:24:04'!
addTransactions: aListOfTransactions 
											
	|newList|
	newList := String new.
	
 	(aListOfTransactions)do:[:aTransaction | newList := newList,(String streamContents: [:str | str 
																								nextPutAll: '|';
																								nextPutAll: (aTransaction key);
																								nextPutAll: '|';
																								nextPutAll: (aTransaction value printString )])].
																						
	^newList.					
! !

!RestAPI methodsFor: 'listing purchases tests' stamp: 'MEV 6/24/2019 15:35:53'!
listPurchasesWithId: aClientID andPassword: aPassword andValidateUsing: idAndPasswordDB
	
	|clientTransactions list total|
	
	idAndPasswordDB logInWithId: aClientID andPassword: aPassword.
	
	clientTransactions := self transactionsOfClient: aClientID.
	
	total := 0.00.
	list := '0'.
	(clientTransactions)do:[:aTransactionID | |aCashier| 
												aCashier := cashiers at: aTransactionID  .
												list := list,(self addTransactions: (aCashier transactions) ).
												total := total + (aCashier totalAmount). 
												].
	^(list,'|'),(total printString).! !

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

!classDefinition: 'RestAPI class' category: #'TusLibros.com'!
RestAPI class
	instanceVariableNames: ''!

!RestAPI class methodsFor: 'initialize' stamp: 'MEV 6/24/2019 12:41:43'!
withMP: aMerchantProcessor

	^self new initializeWithMerchantProcessor: aMerchantProcessor .! !


!RestAPI class methodsFor: 'error messages' stamp: 'MEV 6/24/2019 13:13:58'!
creditCardNameTooLongErrorSignal

	^'1|Invalid credit card name'.! !

!RestAPI class methodsFor: 'error messages' stamp: 'MEV 6/24/2019 13:14:05'!
creditCardWithWrongNumberErrorSignal

	^'1|Incorrect credit card number'! !

!RestAPI class methodsFor: 'error messages' stamp: 'MEV 6/24/2019 12:03:40'!
emptyCartErrorMessage

	^'1|Can not list an empty cart'.! !

!RestAPI class methodsFor: 'error messages' stamp: 'MEV 6/24/2019 16:00:52'!
expiredCartErrorMessage

	^'1|The cart has expired'.! !

!RestAPI class methodsFor: 'error messages' stamp: 'MEV 6/24/2019 14:04:05'!
expiredCreditCardErrorSignal

	^'1|Expired credit card'! !

!RestAPI class methodsFor: 'error messages' stamp: 'MEV 6/24/2019 11:32:52'!
invalidAmountOfBooksErrorMessage

	^'1|Invalid amount of books'.! !

!RestAPI class methodsFor: 'error messages' stamp: 'MEV 6/24/2019 11:26:39'!
invalidBookErrorMessage

	^'1|Invalid book'.! !

!RestAPI class methodsFor: 'error messages' stamp: 'MEV 6/24/2019 11:22:23'!
invalidCartErrorMessage

	^'1|Invalid cart ID '.! !

!RestAPI class methodsFor: 'error messages' stamp: 'MEV 6/24/2019 11:03:51'!
invalidClientIDOrPasswordErrorMessage

	^'1|Invalid client ID or password'.! !

!RestAPI class methodsFor: 'error messages' stamp: 'MEV 6/24/2019 14:49:33'!
noPurchasesErrorMessage

	^'1|No purchases were made'.! !


!classDefinition: #ShoppingCart category: #'TusLibros.com'!
Object subclass: #ShoppingCart
	instanceVariableNames: 'cartContents catalog'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'TusLibros.com'!

!ShoppingCart methodsFor: 'adding to cart' stamp: 'MEV 6/10/2019 08:45:50'!
add:aBooksISBN withAmount: anAmount
	
	(anAmount strictlyPositive )ifFalse: [self error: ShoppingCart invalidAmountOfBooksErrorSignal ].
	catalog at: aBooksISBN ifAbsent: [self error: ShoppingCart canNotAddBooksThatAreNotFromTheCatalogErrorSignal .].
	
	cartContents add:(aBooksISBN -> anAmount ).! !


!ShoppingCart methodsFor: 'testing' stamp: 'MEV 6/3/2019 11:18:19'!
amountOf: aBooksISBN

	|amount|
	
	amount:=cartContents detect: [:aBookAndAmount | (aBookAndAmount key) = aBooksISBN ].
	^(amount value).! !

!ShoppingCart methodsFor: 'testing' stamp: 'MEV 6/3/2019 11:22:37'!
amountOfBooksInCart

	^cartContents sum: [:aBookAndAmount | aBookAndAmount value ].! !

!ShoppingCart methodsFor: 'testing' stamp: 'MEV 6/3/2019 11:16:20'!
hasInCart: aBooksISBN

	^cartContents anySatisfy: [:aBookAndAmount | (aBookAndAmount key) = aBooksISBN ].! !

!ShoppingCart methodsFor: 'testing' stamp: 'MEV 6/3/2019 00:58:42'!
isEmpty

	^cartContents isEmpty.! !


!ShoppingCart methodsFor: 'cart contents' stamp: 'MEV 6/3/2019 11:24:24'!
cartContents

	^cartContents ! !


!ShoppingCart methodsFor: 'catalog' stamp: 'MEV 6/10/2019 09:55:02'!
catalog

	^catalog .! !


!ShoppingCart methodsFor: 'initialize' stamp: 'MEV 6/3/2019 10:53:43'!
initializeWithCatalog: aCatalog 
	
	catalog := aCatalog.
	cartContents := OrderedCollection new.! !

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

!classDefinition: 'ShoppingCart class' category: #'TusLibros.com'!
ShoppingCart class
	instanceVariableNames: ''!

!ShoppingCart class methodsFor: 'error signals' stamp: 'MEV 6/3/2019 10:58:14'!
canNotAddBooksThatAreNotFromTheCatalogErrorSignal

	^'Can not add books thar are not from the catalog.'.! !

!ShoppingCart class methodsFor: 'error signals' stamp: 'MEV 6/3/2019 11:49:14'!
cartIsEmptyErrorSignal

	^'The cart is empty.'.! !

!ShoppingCart class methodsFor: 'error signals' stamp: 'MEV 6/3/2019 11:08:31'!
invalidAmountOfBooksErrorSignal

	^'Invalid amount of books.'.! !


!ShoppingCart class methodsFor: 'class initialization' stamp: 'MEV 6/3/2019 10:52:37'!
createCartwithCatalog: aCatalog 
	
	^self new initializeWithCatalog: aCatalog.! !


!classDefinition: #StoreTestObjects category: #'TusLibros.com'!
Object subclass: #StoreTestObjects
	instanceVariableNames: 'today debitBehavior'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'TusLibros.com'!

!StoreTestObjects methodsFor: 'date' stamp: 'MEV 6/24/2019 09:57:45'!
monthOfYear

	^GregorianMonthOfYear yearNumber: (today yearNumber) monthNumber: today monthIndex ! !

!StoreTestObjects methodsFor: 'date' stamp: 'MEV 6/24/2019 09:41:12'!
today

	^today.! !


!StoreTestObjects methodsFor: 'credit card' stamp: 'MEV 6/24/2019 09:48:34'!
expiredCreditCard

	^CreditCard ccn: '1234123412341234' 
				cced: (GregorianMonthOfYear yearNumber: (today yearNumber -1) monthNumber: today monthIndex) 
				cco: 'Juan'! !

!StoreTestObjects methodsFor: 'credit card' stamp: 'MEV 6/24/2019 10:00:50'!
notExpiredCreditCard

	^CreditCard ccn: '1234123412341234' 
				cced: self monthOfYear
				cco: 'Juan'! !

!StoreTestObjects methodsFor: 'credit card' stamp: 'MEV 6/24/2019 10:00:39'!
wrongNameCreditCard

	^CreditCard ccn: '1234123412341234123' 
				cced: self monthOfYear
				cco: 'Juan es el propietario de esta tarjeta de credito'! !

!StoreTestObjects methodsFor: 'credit card' stamp: 'MEV 6/24/2019 10:00:45'!
wrongNumberCreditCard

	^CreditCard ccn: '1234123412341234123' 
				cced: self monthOfYear
				cco: 'Juan'! !


!StoreTestObjects methodsFor: 'initialize' stamp: 'MEV 6/24/2019 12:38:47'!
initialize

	today := DateAndTime now.
	debitBehavior :=  [ :anAmount :aCreditCard | ].! !


!StoreTestObjects methodsFor: 'cart' stamp: 'MEV 6/24/2019 10:09:39'!
createAnotherCart

	|aCatalog|
	
	aCatalog := Dictionary new.
	aCatalog at: 'ISBN1' put:49.333.
	
	^ShoppingCart createCartwithCatalog: aCatalog.! !

!StoreTestObjects methodsFor: 'cart' stamp: 'MEV 6/24/2019 10:07:49'!
createCart

	|aCatalog|
	
	aCatalog := Dictionary new.
	aCatalog at: 'ISBN1' put:50.
	
	^ShoppingCart createCartwithCatalog: aCatalog.! !


!StoreTestObjects methodsFor: 'merchant processor' stamp: 'MEV 6/24/2019 12:38:13'!
checkCreditCard: aCreditCard withTransactionAmount: anAmount

	^debitBehavior value: anAmount value: aCreditCard ! !

!StoreTestObjects methodsFor: 'merchant processor' stamp: 'MEV 6/24/2019 12:39:19'!
debitBehavior: aBehavior

	debitBehavior := aBehavior .! !
